
from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class IAMUserRootAccessKeys(BaseResourceNegativeValueCheck):

    def __init__(self):
        name = "Ensure IAM root user doesnt have Access keys"
        id = "CKV_AWS_348"
        supported_resources = ['aws_iam_access_key']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "user"

    def get_forbidden_values(self):
        return ["root"]


check = IAMUserRootAccessKeys()



from __future__ import annotations

import json
import re
from re import Pattern
from typing import Any, TYPE_CHECKING, Optional

from detect_secrets.util.filetype import FileType
from detect_secrets.plugins.keyword import DENYLIST
from detect_secrets.plugins.keyword import AFFIX_REGEX
from detect_secrets.plugins.keyword import CLOSING
from detect_secrets.plugins.keyword import OPTIONAL_WHITESPACE
from detect_secrets.plugins.keyword import QUOTE
from detect_secrets.plugins.keyword import SECRET

from checkov.secrets.parsers.terraform.multiline_parser import terraform_multiline_parser
from checkov.secrets.parsers.terraform.single_line_parser import terraform_single_line_parser
from checkov.secrets.parsers.yaml.multiline_parser import yml_multiline_parser
from checkov.secrets.parsers.json.multiline_parser import json_multiline_parser

if TYPE_CHECKING:
    from checkov.secrets.parsers.multiline_parser import BaseMultiLineParser
    from detect_secrets.core.potential_secret import PotentialSecret
    from detect_secrets.util.code_snippet import CodeSnippet
    from detect_secrets.plugins.base import BasePlugin

MAX_KEYWORD_LIMIT = 500

DENY_LIST_REGEX = r'|'.join(DENYLIST)
# Support for suffix after keyword i.e. password_secure = "value"
DENY_LIST_REGEX2 = r'({denylist}){suffix}'.format(
    denylist=DENY_LIST_REGEX,
    suffix=AFFIX_REGEX,
)

KEY = r'{words}({closing})?'.format(
    words=AFFIX_REGEX,
    closing=CLOSING,
)

FOLLOWED_BY_COLON_VALUE_KEYWORD_REGEX = re.compile(
    # e.g. var: MY_PASSWORD_123
    r'{whitespace}({key})?:{whitespace}({quote}?){words}{denylist}({closing})?(\3)'.format(
        key=KEY,
        whitespace=OPTIONAL_WHITESPACE,
        quote=QUOTE,
        words=AFFIX_REGEX,
        denylist=DENY_LIST_REGEX2,
        closing=CLOSING,
    ),
    flags=re.IGNORECASE,
)

QUOTES_REQUIRED_FOLLOWED_BY_COLON_VALUE_KEYWORD_REGEX = re.compile(
    # e.g. var: MY_PASSWORD_123
    r'{whitespace}"({key})?":{whitespace}("?){words}{denylist}({closing})?(\3)'.format(
        key=KEY,
        whitespace=OPTIONAL_WHITESPACE,
        words=AFFIX_REGEX,
        denylist=DENY_LIST_REGEX2,
        closing=CLOSING,
    ),
    flags=re.IGNORECASE,
)

FOLLOWED_BY_COLON_VALUE_SECRET_REGEX = re.compile(
    # e.g. var: Zmlyc3Rfc2VjcmV0X2hlcmVfd2hvYV9tdWx0aWxsaW5lX3Nob3VsZF93b3JrXzE==
    r'{whitespace}({key})?:{whitespace}({quote}?)({secret})(\3)'.format(
        key=KEY,
        whitespace=OPTIONAL_WHITESPACE,
        quote=QUOTE,
        secret=SECRET,
    ),
    flags=re.IGNORECASE,
)

QUOTES_REQUIRED_FOLLOWED_BY_COLON_VALUE_SECRET_REGEX = re.compile(
    # e.g. var: Zmlyc3Rfc2VjcmV0X2hlcmVfd2hvYV9tdWx0aWxsaW5lX3Nob3VsZF93b3JrXzE==
    r'{whitespace}"({key})?":{whitespace}("?)({secret})(\3)'.format(
        key=KEY,
        whitespace=OPTIONAL_WHITESPACE,
        secret=SECRET,
    ),
    flags=re.IGNORECASE,
)

FOLLOWED_BY_EQUAL_VALUE_KEYWORD_REGEX = re.compile(
    # e.g. var = MY_PASSWORD_123
    r'{whitespace}({key})?={whitespace}({quote}?){words}{denylist}({closing})?(\3)'.format(
        key=KEY,
        whitespace=OPTIONAL_WHITESPACE,
        quote=QUOTE,
        words=AFFIX_REGEX,
        denylist=DENY_LIST_REGEX2,
        closing=CLOSING,
    ),
    flags=re.IGNORECASE,
)

FOLLOWED_BY_EQUAL_VALUE_SECRET_REGEX = re.compile(
    # e.g. var = Zmlyc3Rfc2VjcmV0X2hlcmVfd2hvYV9tdWx0aWxsaW5lX3Nob3VsZF93b3JrXzE==
    r'{whitespace}({key})?={whitespace}({quote}?)({secret})(\3)'.format(
        key=KEY,
        whitespace=OPTIONAL_WHITESPACE,
        quote=QUOTE,
        secret=SECRET,
    ),
    flags=re.IGNORECASE,
)

ALLOW_LIST = ('secretsmanager', "secretName", "secret_name", "creation_token")  # can add more keys like that
ALLOW_LIST_REGEX = r'|'.join(ALLOW_LIST)
# Support for suffix of function name i.e "secretsmanager:GetSecretValue"
CAMEL_CASE_NAMES = r'[A-Z]([A-Z0-9]*[a-z][a-z0-9]*[A-Z]|[a-z0-9]*[A-Z][A-Z0-9]*[a-z])[A-Za-z0-9]*'
FUNCTION_CALL_AFTER_KEYWORD_REGEX = re.compile(r'({allowlist})\s*(:|=)\s*{suffix}'.format(
    allowlist=ALLOW_LIST_REGEX,
    suffix=AFFIX_REGEX,
))

#  if the current regex is not enough, can add more regexes to check

YML_PAIR_VALUE_KEYWORD_REGEX_TO_GROUP = {
    FOLLOWED_BY_COLON_VALUE_KEYWORD_REGEX: 4,
}

YML_PAIR_VALUE_SECRET_REGEX_TO_GROUP = {
    FOLLOWED_BY_COLON_VALUE_SECRET_REGEX: 4,
}

JSON_PAIR_VALUE_KEYWORD_REGEX_TO_GROUP = {
    QUOTES_REQUIRED_FOLLOWED_BY_COLON_VALUE_KEYWORD_REGEX: 4,
}

JSON_PAIR_VALUE_SECRET_REGEX_TO_GROUP = {
    QUOTES_REQUIRED_FOLLOWED_BY_COLON_VALUE_SECRET_REGEX: 4,
}

TERRAFORM_PAIR_VALUE_KEYWORD_REGEX_TO_GROUP = {
    FOLLOWED_BY_EQUAL_VALUE_KEYWORD_REGEX: 4,
}

TERRAFORM_PAIR_VALUE_SECRET_REGEX_TO_GROUP = {
    FOLLOWED_BY_EQUAL_VALUE_SECRET_REGEX: 4,
}

REGEX_VALUE_KEYWORD_BY_FILETYPE = {
    FileType.YAML: YML_PAIR_VALUE_KEYWORD_REGEX_TO_GROUP,
    FileType.JSON: JSON_PAIR_VALUE_KEYWORD_REGEX_TO_GROUP,
    FileType.TERRAFORM: TERRAFORM_PAIR_VALUE_KEYWORD_REGEX_TO_GROUP,
}

REGEX_VALUE_SECRET_BY_FILETYPE = {
    FileType.YAML: YML_PAIR_VALUE_SECRET_REGEX_TO_GROUP,
    FileType.JSON: JSON_PAIR_VALUE_SECRET_REGEX_TO_GROUP,
    FileType.TERRAFORM: TERRAFORM_PAIR_VALUE_SECRET_REGEX_TO_GROUP,
}

SINGLE_LINE_PARSER = {
    FileType.TERRAFORM: terraform_single_line_parser,
}

MULTILINE_PARSERS = {
    FileType.YAML: (
        (FileType.YAML, yml_multiline_parser),
    ),
    FileType.JSON: (
        (FileType.JSON, json_multiline_parser),
    ),
    FileType.TERRAFORM: (
        (FileType.TERRAFORM, terraform_multiline_parser),
        (FileType.JSON, json_multiline_parser),
        (FileType.YAML, yml_multiline_parser),
    ),
}


def remove_fp_secrets_in_keys(detected_secrets: set[PotentialSecret], line: str, is_code_file: bool = False) -> None:
    formatted_line = line.replace('"', '').replace("'", '')
    secrets_to_remove = set()
    for detected_secret in detected_secrets:
        if not detected_secret.secret_value:
            continue
        # Found keyword prefix as potential secret
        if formatted_line.startswith(detected_secret.secret_value):
            secrets_to_remove.add(detected_secret)
        # found a function name at the end of the line
        if formatted_line and FUNCTION_CALL_AFTER_KEYWORD_REGEX.search(formatted_line):
            secrets_to_remove.add(detected_secret)
        # secret value is substring of keywork
        if is_code_file and FOLLOWED_BY_EQUAL_VALUE_KEYWORD_REGEX.search(formatted_line):
            key, value = line.split("=", 1)
            if detected_secret.secret_value in key and detected_secret.secret_value in value:
                secrets_to_remove.add(detected_secret)
    detected_secrets -= secrets_to_remove


def format_reducing_noise_secret(string: str) -> str:
    return json.dumps(string)


def extract_from_string(pattern: dict[Pattern[str], int] | None, string: str) -> set[str]:
    matches: set[str] = set()
    if not pattern:
        return matches
    for value_regex, group_number in pattern.items():
        match = value_regex.search(string)
        if match:
            matches |= {match.group(group_number).rstrip('\n')}
    return matches


def detect_secret(
        scanners: tuple[BasePlugin, ...],
        filename: str,
        line: str,
        line_number: int = 0,
        is_multiline: Optional[bool] = None,
        **kwargs: Any,
) -> set[PotentialSecret]:
    for scanner in scanners:
        matches = scanner.analyze_line(filename, line, line_number, **kwargs)
        if matches:
            if is_multiline:
                mark_set_multiline(matches)
            return matches
    return set()


def analyze_multiline_keyword_combinator(
        filename: str,
        scanners: tuple[BasePlugin, ...],
        multiline_parser: BaseMultiLineParser,
        line_number: int,
        context: CodeSnippet | None = None,
        raw_context: CodeSnippet | None = None,
        value_pattern: dict[Pattern[str], int] | None = None,
        secret_pattern: dict[Pattern[str], int] | None = None,
        is_added: bool = False,
        is_removed: bool = False,
        **kwargs: Any,
) -> set[PotentialSecret]:
    secrets: set[PotentialSecret] = set()
    if context is None or raw_context is None:
        return secrets
    value_secrets = extract_from_string(pattern=secret_pattern, string=context.target_line)
    for possible_secret in value_secrets:
        secret_adjust = format_reducing_noise_secret(possible_secret)

        potential_secrets = detect_secret(
            scanners=scanners,
            filename=filename,
            line=secret_adjust,
            line_number=line_number,
            is_added=is_added,
            is_removed=is_removed,
            is_multiline=True,  # always true because we check here for multiline
            kwargs=kwargs
        )

        if potential_secrets:
            possible_keywords: set[str] = set()
            backwards_range = range(context.target_index - 1, -1, -1)
            forward_range = range(context.target_index + 1, len(context.lines))

            possible_keywords |= multiline_parser.get_lines_from_same_object(
                search_range=forward_range,
                context=context,
                raw_context=raw_context,
                line_length_limit=MAX_KEYWORD_LIMIT)
            possible_keywords |= multiline_parser.get_lines_from_same_object(
                search_range=backwards_range,
                context=context,
                raw_context=raw_context,
                line_length_limit=MAX_KEYWORD_LIMIT)

            for other_value in possible_keywords:
                if extract_from_string(pattern=value_pattern, string=other_value):
                    secrets |= potential_secrets
                    break
    return secrets


def mark_set_multiline(secrets: set[PotentialSecret]) -> None:
    for sec in secrets:
        sec.is_multiline = True



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class DropCapabilities(BaseResourceCheck):

    def __init__(self):
        # CIS-1.3 1.7.7
        # CIS-1.5 5.2.7
        # NET_RAW allows a process to spy on packets on its network
        name = "Minimize the admission of containers with the NET_RAW capability"
        id = "CKV_K8S_28"

        supported_resources = ('kubernetes_pod', 'kubernetes_pod_v1',
                               'kubernetes_deployment', 'kubernetes_deployment_v1', )
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        if "spec" not in conf:
            self.evaluated_keys = [""]
            return CheckResult.FAILED
        spec = conf['spec'][0]
        if not spec:
            return CheckResult.UNKNOWN

        evaluated_keys_path = "spec"

        template = spec.get("template")
        if template and isinstance(template, list):
            template = template[0]
            template_spec = template.get("spec")
            if template_spec and isinstance(template_spec, list):
                spec = template_spec[0]
                evaluated_keys_path = f'{evaluated_keys_path}/[0]/template/[0]/spec'

        if spec.get("container"):
            containers = spec.get("container")

            for idx, container in enumerate(containers):
                if not isinstance(container, dict):
                    return CheckResult.UNKNOWN
                dropped = False
                if container.get("security_context") and isinstance(container.get("security_context"), list):
                    context = container.get("security_context")[0]
                    if context.get("capabilities") and isinstance(context.get("capabilities"), list):
                        capabilities = context.get("capabilities")[0]
                        if isinstance(capabilities, dict) and capabilities.get("drop") and isinstance(capabilities.get("drop"), list):
                            drops = capabilities.get("drop")[0]
                            for drop in drops:
                                if drop in ["ALL", "NET_RAW"]:
                                    dropped = True
                            if not dropped:
                                return CheckResult.FAILED
                        else:
                            self.evaluated_keys = [f"{evaluated_keys_path}/[0]/container/{idx}/security_context/[0]/capabilities"]
                            return CheckResult.FAILED
                    else:
                        self.evaluated_keys = [f"{evaluated_keys_path}/[0]/container/{idx}/security_context"]
                        return CheckResult.FAILED
                else:
                    self.evaluated_keys = [f"{evaluated_keys_path}/[0]/container/{idx}"]
                    return CheckResult.FAILED
            return CheckResult.PASSED
        return CheckResult.FAILED


check = DropCapabilities()



from __future__ import annotations

import logging
import subprocess  # nosec


def get_kustomize_version(kustomize_command: str) -> str | None:
    try:
        proc = subprocess.run([kustomize_command, "version"], capture_output=True)  # nosec
        version_output = proc.stdout.decode("utf-8")

        if "Version:" in version_output:
            # version <= 4 output looks like '{Version:kustomize/v4.5.7 GitCommit:...}\n'
            kustomize_version = version_output[version_output.find("/") + 1 : version_output.find("G") - 1]
        elif version_output.startswith("v"):
            # version >= 5 output looks like 'v5.0.0\n'
            kustomize_version = version_output.rstrip("\n")
        else:
            return None

        return kustomize_version
    except Exception:
        logging.debug(f"An error occurred testing the {kustomize_command} command:", exc_info=True)

    return None


def get_kubectl_version(kubectl_command: str) -> float | None:
    try:
        proc = subprocess.run([kubectl_command, "version", "--client=true"], capture_output=True)  # nosec
        version_output = proc.stdout.decode("utf-8")

        if "Client Version:" in version_output:
            if "Major:" in version_output and "Minor:" in version_output:
                # version <= 1.27 output looks like 'Client Version: version.Info{Major:"1", Minor:"27", GitVersion:...}\n...'
                kubectl_version_major = version_output.split("\n")[0].split('Major:"')[1].split('"')[0]
                kubectl_version_minor = version_output.split("\n")[0].split('Minor:"')[1].split('"')[0]
            else:
                # version >= 1.28 output looks like 'Client Version: v1.28.0\n...'
                kubectl_version_str = version_output.split("\n")[0].replace("Client Version: v", "")
                kubectl_version_major, kubectl_version_minor, *_ = kubectl_version_str.split(".")
            kubectl_version = float(f"{kubectl_version_major}.{kubectl_version_minor}")

            return kubectl_version
    except Exception:
        logging.debug(f"An error occurred testing the {kubectl_command} command:", exc_info=True)

    return None



from dataclasses import dataclass
from typing import TYPE_CHECKING

from checkov.common.graph.graph_builder.graph_components.block_types import BlockType as CommonBlockType

if TYPE_CHECKING:
    from typing_extensions import Literal


@dataclass
class BlockType(CommonBlockType):
    PARAMETER: 'Literal["parameter"]' = "parameter"




from checkov.arm.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class AppServiceHttpLoggingEnabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that App service enables HTTP logging"
        id = "CKV_AZURE_63"
        supported_resources = ["Microsoft.Web/sites/config"]
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "properties/httpLoggingEnabled"

    def get_expected_value(self) -> bool:
        return True


check = AppServiceHttpLoggingEnabled()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class VMAgentIsInstalled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure VM agent is installed"
        id = "CKV_AZURE_179"
        supported_resources = (
            "azurerm_windows_virtual_machine",
            "azurerm_windows_virtual_machine_scale_set",
            "azurerm_linux_virtual_machine_scale_set",
            "azurerm_linux_virtual_machine",
        )
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources, missing_block_result=CheckResult.PASSED
        )

    def get_inspected_key(self) -> str:
        return "provision_vm_agent"


check = VMAgentIsInstalled()



from __future__ import annotations

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from typing import Any


class LBNetworkPrivate(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure Load Balancer isn't exposed to the internet"
        id = "CKV_NCP_16"
        supported_resources = ("ncloud_lb",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "network_type"

    def get_expected_value(self) -> Any:
        return "PRIVATE"


check = LBNetworkPrivate()



from __future__ import annotations

from typing import Any

from checkov.ansible.checks.base_ansible_task_value_check import BaseAnsibleTaskValueCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class EC2PublicIP(BaseAnsibleTaskValueCheck):
    def __init__(self) -> None:
        name = "EC2 instance should not have public IP."
        id = "CKV_AWS_88"
        super().__init__(
            name=name,
            id=id,
            categories=(CheckCategories.NETWORKING,),
            supported_modules=("amazon.aws.ec2_instance", "ec2_instance"),
        )

    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        if not conf.get("image_id") and not conf.get("image"):
            # if 'image_id' or 'image' are not set, then an already running instance is targeted
            return CheckResult.UNKNOWN, self.entity_conf

        return super().scan_conf(conf=conf)

    def get_inspected_key(self) -> str:
        return "network/assign_public_ip"

    def get_expected_value(self) -> Any:
        return False


check = EC2PublicIP()



from __future__ import annotations

from checkov.github.base_github_branch_security import BranchSecurity


class GithubBranchCommitSignature(BranchSecurity):
    def __init__(self) -> None:
        name = "Ensure GitHub branch protection rules requires signed commits"
        id = "CKV_GITHUB_4"
        super().__init__(
            name=name,
            id=id
        )

    def get_evaluated_keys(self) -> list[str]:
        return ['required_signatures/enabled']


check = GithubBranchCommitSignature()



from __future__ import annotations

from typing import Any
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.checks.enums import BlockType
from checkov.openapi.checks.resource.v2.BaseOpenapiCheckV2 import BaseOpenapiCheckV2


class Oauth2SecurityRequirement(BaseOpenapiCheckV2):
    def __init__(self) -> None:
        id = "CKV_OPENAPI_2"
        name = "Ensure that if the security scheme is not of type 'oauth2', the array value must be empty - version 2.0 files"
        categories = (CheckCategories.API_SECURITY,)
        supported_resources = ('security',)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_resources,
            block_type=BlockType.DOCUMENT,
        )

    def scan_openapi_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]]:
        security_values = conf.get("security") or [{}]
        security_definitions = conf.get("securityDefinitions") or {}
        non_oauth2_keys = []

        for auth_key, auth_dict in security_definitions.items():
            if self.is_start_end_line(auth_key):
                continue
            auth_type = auth_dict.get("type")
            if auth_type.lower() != "oauth2":
                non_oauth2_keys.append(auth_key)

        for auth_dict in security_values:
            if not isinstance(auth_dict, dict):
                return CheckResult.UNKNOWN, conf
            for key, auth_list in auth_dict.items():
                if self.is_start_end_line(key):
                    continue
                if key in non_oauth2_keys and auth_list:
                    return CheckResult.FAILED, auth_dict

        return CheckResult.PASSED, conf


check = Oauth2SecurityRequirement()



from __future__ import annotations

from typing import Iterable, TYPE_CHECKING

from checkov.common.checks.base_check import BaseCheck
from checkov.openapi.checks.registry import openapi_registry as registry

if TYPE_CHECKING:
    from checkov.common.models.enums import CheckCategories


class BaseOpenapiCheck(BaseCheck):
    def __init__(self, name: str, id: str, categories: Iterable[CheckCategories], supported_entities: Iterable[str],
                 block_type: str, path: str | None = None, guideline: str | None = None) -> None:
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_entities,
            block_type=block_type,
            guideline=guideline,
        )
        self.path = path
        registry.register(self)

    def is_start_end_line(self, conf: str) -> bool:
        return conf in {'__startline__', '__endline__'}



from __future__ import annotations

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class LBTargetGroupUsingHTTPS(BaseResourceCheck):

    def __init__(self):
        name = "Ensure Load Balancer Target Group is not using HTTP"
        id = "CKV_NCP_15"
        supported_resources = ("ncloud_lb_target_group",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if "protocol" in conf.keys():
            if conf.get("protocol") != ['HTTP']:
                return CheckResult.PASSED
        return CheckResult.FAILED


check = LBTargetGroupUsingHTTPS()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from typing import Any


class MongoDBInsideVPC(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure MongoDB is deployed inside a VPC"
        id = "CKV_ALI_41"
        supported_resources = ("alicloud_mongodb_instance",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "network_type"

    def get_expected_value(self) -> Any:
        return "VPC"


check = MongoDBInsideVPC()



from __future__ import annotations

import logging
import os
from collections import defaultdict
from functools import partial
from pathlib import Path
from typing import List, Optional, Union, Any, Dict, overload

from typing_extensions import TypedDict

import checkov.terraform.graph_builder.foreach.consts
from checkov.common.graph.graph_builder import Edge
from checkov.common.graph.graph_builder import reserved_attribute_names
from checkov.common.graph.graph_builder.graph_components.attribute_names import CustomAttributes
from checkov.common.graph.graph_builder.local_graph import LocalGraph
from checkov.common.graph.graph_builder.utils import calculate_hash, join_trimmed_strings, filter_sub_keys
from checkov.common.runners.base_runner import strtobool
from checkov.common.util.data_structures_utils import pickle_deepcopy
from checkov.common.util.type_forcers import force_int
from checkov.terraform.graph_builder.foreach.builder import ForeachBuilder
from checkov.terraform.graph_builder.variable_rendering.vertex_reference import TerraformVertexReference
from checkov.terraform.modules.module_objects import TFModule
from checkov.terraform.context_parsers.registry import parser_registry
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.graph_components.blocks import TerraformBlock
from checkov.terraform.graph_builder.graph_components.generic_resource_encryption import ENCRYPTION_BY_RESOURCE_TYPE
from checkov.terraform.graph_builder.graph_components.module import Module
from checkov.terraform.graph_builder.utils import (
    get_attribute_is_leaf,
    get_referenced_vertices_in_value,
    attribute_has_nested_attributes, remove_index_pattern_from_str,
)
from checkov.terraform.graph_builder.utils import is_local_path
from checkov.terraform.graph_builder.variable_rendering.renderer import TerraformVariableRenderer


MODULE_RESERVED_ATTRIBUTES = ("source", "version")
CROSS_VARIABLE_EDGE_PREFIX = '[cross-variable] '


class Undetermined(TypedDict):
    module_vertex_id: int
    attribute_name: str
    variable_vertex_id: int


class TerraformLocalGraph(LocalGraph[TerraformBlock]):
    def __init__(self, module: Module) -> None:
        super().__init__()
        self.vertices: list[TerraformBlock] = []
        self.module = module
        self.map_path_to_module: Dict[str, List[int]] = {}
        self.relative_paths_cache: dict[tuple[str, str], str] = {}
        self.abspath_cache: Dict[str, str] = {}
        self.dirname_cache: Dict[str, str] = {}
        self.vertices_by_module_dependency_by_name: Dict[TFModule | None, Dict[str, Dict[str, List[int]]]] = defaultdict(partial(defaultdict, partial(defaultdict, list)))  # type:ignore[arg-type]
        self.vertices_by_module_dependency: Dict[TFModule | None, Dict[str, List[int]]] = defaultdict(partial(defaultdict, list))  # type:ignore[arg-type]
        self.enable_foreach_handling = strtobool(os.getenv('CHECKOV_ENABLE_FOREACH_HANDLING', 'True'))
        self.enable_modules_foreach_handling = strtobool(os.getenv('CHECKOV_ENABLE_MODULES_FOREACH_HANDLING', 'True'))
        self.foreach_blocks: Dict[str, List[int]] = {BlockType.RESOURCE: [], BlockType.MODULE: []}

    def build_graph(self, render_variables: bool) -> None:
        self._create_vertices()
        logging.info(f"[TerraformLocalGraph] created {len(self.vertices)} vertices")
        self._build_edges()
        logging.info(f"[TerraformLocalGraph] created {len(self.edges)} edges")
        if (self.enable_foreach_handling or self.enable_modules_foreach_handling) \
                and (self.foreach_blocks[BlockType.RESOURCE] or self.foreach_blocks[BlockType.MODULE]):
            try:
                logging.info('[TerraformLocalGraph] start handling foreach')
                foreach_builder = ForeachBuilder(self)
                foreach_builder.handle(self.foreach_blocks)
                self._arrange_graph_data()
                self._build_edges()
                logging.info(f"[TerraformLocalGraph] finished handling foreach values with {len(self.vertices)} vertices and {len(self.edges)} edges")
            except Exception as e:
                logging.info(f'Failed to process foreach handling, error: {str(e)}')

        self.calculate_encryption_attribute(ENCRYPTION_BY_RESOURCE_TYPE)
        if render_variables:
            logging.info(f"Rendering variables, graph has {len(self.vertices)} vertices and {len(self.edges)} edges")
            renderer = TerraformVariableRenderer(self)
            renderer.render_variables_from_local_graph()
            self.update_vertices_fields()
            if strtobool(os.getenv("CHECKOV_EXPERIMENTAL_CROSS_VARIABLE_EDGES", "True")):
                # experimental flag on building cross variable edges for terraform graph
                logging.info("Building cross variable edges")
                edges_count = len(self.edges)
                self._build_cross_variable_edges()
                logging.info(f"Found {len(self.edges) - edges_count} cross variable edges")
        else:
            self.update_vertices_fields()

    def update_vertices_fields(self) -> None:
        # Important to keep those 2 functions together, as the first affects the calculation of the second
        self._update_vertices_breadcrumbs_and_module_connections()
        self._update_nested_modules_address()

    def _create_vertices(self) -> None:
        logging.info("Creating vertices")
        self.vertices = [None] * len(self.module.blocks)  # type:ignore[list-item]  # are correctly set in the next lines
        for i, block in enumerate(self.module.blocks):
            self.vertices[i] = block
            self._add_block_data_to_graph(i, block)
            if self.enable_foreach_handling and (
                    checkov.terraform.graph_builder.foreach.consts.FOREACH_STRING in block.attributes or checkov.terraform.graph_builder.foreach.consts.COUNT_STRING in block.attributes) \
                    and block.block_type in (BlockType.MODULE, BlockType.RESOURCE):
                self.foreach_blocks[block.block_type].append(i)

    def _add_block_data_to_graph(self, idx: int, block: TerraformBlock) -> None:
        self.vertices_by_block_type[block.block_type].append(idx)
        self.vertices_block_name_map[block.block_type][block.name].append(idx)

        if block.block_type == BlockType.MODULE:
            # map between file paths and module vertices indexes from that file
            self.map_path_to_module.setdefault(block.path, []).append(idx)

        self.vertices_by_module_dependency[block.source_module_object][block.block_type].append(idx)
        self.vertices_by_module_dependency_by_name[block.source_module_object][block.block_type][block.name].append(idx)

        self.in_edges[idx] = []
        self.out_edges[idx] = []

    def _arrange_graph_data(self) -> None:
        # reset all the relevant data
        self.vertices_by_block_type = defaultdict(list)
        self.vertices_block_name_map = defaultdict(partial(defaultdict, list))  # type:ignore[arg-type]
        self.map_path_to_module = {}
        self.vertices_by_module_dependency = defaultdict(partial(defaultdict, list))  # type:ignore[arg-type]
        self.vertices_by_module_dependency_by_name = defaultdict(partial(defaultdict, partial(defaultdict, list)))  # type:ignore[arg-type]
        self.edges = []
        for i in range(len(self.vertices)):
            self.out_edges[i] = []
            self.in_edges[i] = []

        for i, block in enumerate(self.vertices):
            self._add_block_data_to_graph(i, block)

    def _get_aliases(self) -> Dict[str, Dict[str, str]]:
        """
        :return aliases: map between alias names that are found inside the blocks and the block type their aliased to.
        """
        return {
            vertex.name: {CustomAttributes.BLOCK_TYPE: vertex.block_type}
            for vertex in self.vertices
            if "alias" in vertex.attributes
        }

    def get_module_vertices_mapping(self) -> None:
        """
        For each vertex, if it's originated in a module import, add to the vertex the index of the
        matching module vertex as 'source_module'
        """
        for vertex in self.vertices:
            if not vertex.source_module_object:
                continue
            for idx in self.vertices_by_block_type[BlockType.MODULE]:
                if vertex.source_module_object.name != self.vertices[idx].name:
                    continue
                if vertex.source_module_object.path != self.vertices[idx].path:
                    continue
                if vertex.source_module_object.nested_tf_module != self.vertices[idx].source_module_object:
                    continue
                if vertex.source_module_object.foreach_idx != self.vertices[idx].for_each_index:
                    continue
                vertex.source_module.add(idx)
                break
        return

    def _build_edges(self) -> None:
        logging.info("Creating edges")
        self.get_module_vertices_mapping()
        aliases = self._get_aliases()
        resources_types = self.get_resources_types_in_graph()
        for origin_node_index, vertex in enumerate(self.vertices):
            self._build_edges_for_vertex(origin_node_index, vertex, aliases, resources_types)

    def _build_edges_for_vertex(self, origin_node_index: int, vertex: TerraformBlock, aliases: Dict[str, Dict[str, str]],
                                resources_types: List[str], cross_variable_edges: bool = False,
                                referenced_modules: Optional[List[Dict[str, Any]]] = None) -> None:

        attribute_is_leaf = get_attribute_is_leaf(vertex)
        for attribute_key, attribute_value in vertex.attributes.items():
            if attribute_key in reserved_attribute_names or attribute_has_nested_attributes(
                    attribute_key, vertex.attributes, attribute_is_leaf
            ):
                continue
            referenced_vertices = get_referenced_vertices_in_value(
                value=attribute_value,
                aliases=aliases,
                resources_types=resources_types,
            )
            for vertex_reference in referenced_vertices:
                # for certain blocks such as data and resource, the block name is composed from several parts.
                # the purpose of the loop is to avoid not finding the node if the name has several parts
                sub_values = [remove_index_pattern_from_str(sub_value) for sub_value in vertex_reference.sub_parts]
                for i in range(len(sub_values)):
                    reference_name = join_trimmed_strings(char_to_join=".", str_lst=sub_values, num_to_trim=i)
                    source_module_object = vertex.source_module_object
                    if referenced_modules is not None:
                        for module in referenced_modules:
                            referenced_module_idx = module.get("idx")
                            referenced_module_path = module.get("path")
                            if referenced_module_path is None:
                                dest_node_index = -1
                            else:
                                dest_node_index = self._find_vertex_index_relative_to_path(
                                    vertex_reference.block_type, reference_name, referenced_module_path,
                                    referenced_module_idx,
                                    source_module_object=source_module_object
                                )
                            self._create_edge_from_reference(attribute_key, origin_node_index, dest_node_index,
                                                             sub_values, vertex_reference, cross_variable_edges)
                    if vertex.source_module_object:
                        dest_node_index = self._find_vertex_index_relative_to_path(
                            vertex_reference.block_type, reference_name, vertex.path,
                            source_module_object=source_module_object
                        )
                        if dest_node_index == -1:
                            dest_node_index = self._find_vertex_index_relative_to_path(
                                vertex_reference.block_type, reference_name, vertex.path,
                                source_module_object=source_module_object
                            )
                    else:
                        dest_node_index = self._find_vertex_index_relative_to_path(
                            vertex_reference.block_type, reference_name, vertex.path,
                            source_module_object=source_module_object
                        )
                    if dest_node_index > -1 and origin_node_index > -1:
                        self._create_edge_from_reference(attribute_key, origin_node_index, dest_node_index, sub_values,
                                                         vertex_reference, cross_variable_edges)
                        break

        if vertex.block_type == BlockType.MODULE and vertex.attributes.get('source') \
                and isinstance(vertex.attributes['source'][0], str):
            dest_module_path = self._get_dest_module_path(
                curr_module_dir=self.get_dirname(vertex.path),
                dest_module_source=vertex.attributes["source"][0],
                dest_module_version=vertex.attributes.get("version", ["latest"])[0]
            )
            target_variables = self._get_target_variables(vertex, dest_module_path)
            for attribute in vertex.attributes.keys():
                if attribute in MODULE_RESERVED_ATTRIBUTES:
                    continue
                target_variable = next((v for v in target_variables if self.vertices[v].name == attribute), None)
                if target_variable is not None:
                    self.create_edge(target_variable, origin_node_index, "default", cross_variable_edges)
        elif vertex.block_type == BlockType.TF_VARIABLE:
            # Assuming the tfvars file is in the same directory as the variables file (best practice)
            target_variable = 0
            for index in self.vertices_block_name_map.get(BlockType.VARIABLE, {}).get(vertex.name, []):
                if self.get_dirname(self.vertices[index].path) == self.get_dirname(vertex.path):
                    target_variable = index
                    break
            if target_variable:
                self.create_edge(target_variable, origin_node_index, "default", cross_variable_edges)

    def _create_edge_from_reference(self, attribute_key: Any, origin_node_index: int, dest_node_index: int,
                                    sub_values: List[Any], vertex_reference: TerraformVertexReference,
                                    cross_variable_edges: bool) -> None:
        if dest_node_index > -1 and origin_node_index > -1:
            if vertex_reference.block_type == BlockType.MODULE:
                try:
                    self._connect_module(
                        sub_values, attribute_key, self.vertices[dest_node_index],
                        origin_node_index,
                        cross_variable_edges
                    )
                except Exception:
                    logging.warning(
                        f"Module {self.vertices[dest_node_index]} does not have source attribute, skipping"
                    )
            else:
                self.create_edge(origin_node_index, dest_node_index, attribute_key,
                                 cross_variable_edges)

    def _get_target_variables(self, vertex: TerraformBlock, dest_module_path: str) -> list[int]:
        target_path = get_vertex_as_tf_module(vertex)
        return [
            index
            for index in self.vertices_by_module_dependency.get(target_path, {}).get(BlockType.VARIABLE, [])
            if self.get_dirname(self.vertices[index].path) == dest_module_path
        ]

    def _build_cross_variable_edges(self) -> None:
        aliases = self._get_aliases()
        resources_types = self.get_resources_types_in_graph()
        for origin_node_index, referenced_vertices in self.out_edges.items():
            vertex = self.vertices[origin_node_index]
            if vertex.block_type == BlockType.RESOURCE and \
                    any(self.vertices[e.dest].block_type != BlockType.RESOURCE for e in referenced_vertices):
                modules = vertex.breadcrumbs.get(CustomAttributes.SOURCE_MODULE, [])
                self._build_edges_for_vertex(origin_node_index, vertex, aliases, resources_types, True, modules)

    def create_edge(self, origin_vertex_index: int, dest_vertex_index: int, label: str,
                    cross_variable_edges: bool = False) -> bool:
        if origin_vertex_index == dest_vertex_index:
            return False
        edge = Edge(origin_vertex_index, dest_vertex_index, label)
        if cross_variable_edges:
            if self.vertices[dest_vertex_index].block_type != BlockType.RESOURCE or \
                    self.vertices[origin_vertex_index].block_type != BlockType.RESOURCE:
                return False
            if edge in self.out_edges[origin_vertex_index]:
                return False
            edge.label = CROSS_VARIABLE_EDGE_PREFIX + edge.label
            if edge in self.out_edges[origin_vertex_index]:
                return False
        self.edges.append(edge)
        self.out_edges[origin_vertex_index].append(edge)
        self.in_edges[dest_vertex_index].append(edge)
        return True

    def _connect_module(
        self, sub_values: List[str], attribute_key: str, module_node: TerraformBlock, origin_node_index: int, cross_variable_edges: bool = False
    ) -> None:
        """
        :param sub_values: list of sub values of the attribute value.
                            example: given 'instance_type = module.child.myoutput',
                                        then attribute_key = instance_type, sub_values = ['child', 'myoutput']
        :param attribute_key: the name of the attribute that has module block as value
        :param module_node: the graph node of the module

        The function receives a node of a block of type BlockType.Module, and finds all the nodes of blocks that belong to this
        module, and creates edges between them.
        """
        curr_module_dir = self.get_dirname(module_node.path)
        dest_module_source = module_node.attributes["source"][0]
        dest_module_version = module_node.attributes.get("version", ["latest"])[0]
        dest_module_path = self._get_dest_module_path(
            curr_module_dir=curr_module_dir,
            dest_module_source=dest_module_source,
            dest_module_version=dest_module_version
        )

        if len(sub_values) > 1:
            block_name_in_other_module = sub_values[1]
            output_blocks_with_name = self.vertices_block_name_map.get(BlockType.OUTPUT, {}).get(
                block_name_in_other_module, []
            )
            for vertex_index in output_blocks_with_name:
                vertex = self.vertices[vertex_index]
                if self._should_add_edge(vertex, dest_module_path, module_node):
                    added_edge = self.create_edge(origin_node_index, vertex_index, attribute_key, cross_variable_edges)
                    if added_edge:
                        self.vertices[origin_node_index].add_module_connection(attribute_key, vertex_index)
                    break

    def _get_dest_module_path(self, curr_module_dir: str, dest_module_source: str, dest_module_version: str) -> str:
        """
        :param curr_module_dir: current source directory
        :param dest_module_source: the value of module.source
        :return: the real path in the local file system of the dest module
        """
        relative_path_key = (curr_module_dir, dest_module_source)
        if relative_path_key in self.relative_paths_cache:
            return self.relative_paths_cache[relative_path_key]
        elif is_local_path(curr_module_dir, dest_module_source):
            self.relative_paths_cache[relative_path_key] = os.path.abspath(Path(curr_module_dir) / dest_module_source)
            return self.relative_paths_cache[relative_path_key]
        elif (dest_module_source, dest_module_version) in self.module.external_modules_source_map:
            return self.module.external_modules_source_map[(dest_module_source, dest_module_version)]

        # this happens, when we have external modules, which weren't downloaded
        return ""

    def _find_vertex_index_relative_to_path(
        self,
        block_type: str,
        name: str,
        block_path: str,
        relative_module_idx: Optional[int] = None,
        source_module_object: Optional[TFModule] = None,
    ) -> int:
        relative_vertices: list[int] = []
        if relative_module_idx is None:
            module_dependency_by_name_key = source_module_object
        else:
            vertex = self.vertices[relative_module_idx]
            module_dependency_by_name_key = vertex.source_module_object

        # important to use this specific map for big graph performance
        possible_vertices = self.vertices_by_module_dependency_by_name.get(module_dependency_by_name_key, {}).get(block_type, {}).get(name, [])
        for vertex_index in possible_vertices:
            vertex = self.vertices[vertex_index]
            if self.get_dirname(vertex.path) == self.get_dirname(block_path):
                relative_vertices.append(vertex_index)

        if len(relative_vertices) == 1:
            relative_vertex = relative_vertices[0]
        else:
            relative_vertex = self._find_vertex_with_longest_path_match(relative_vertices, block_path)
        return relative_vertex

    def _find_vertex_with_longest_path_match(self, relevant_vertices_indexes: List[int], origin_path: str) -> int:
        vertex_index_with_longest_common_prefix = -1
        longest_common_prefix = ""
        for vertex_index in relevant_vertices_indexes:
            vertex = self.vertices[vertex_index]
            common_prefix = os.path.commonpath([os.path.realpath(vertex.path), os.path.realpath(origin_path)])
            if len(common_prefix) > len(longest_common_prefix):
                vertex_index_with_longest_common_prefix = vertex_index
                longest_common_prefix = common_prefix
        return vertex_index_with_longest_common_prefix

    def get_vertices_hash_codes_to_attributes_map(self) -> Dict[str, Dict[str, Any]]:
        return {vertex.get_hash(): vertex.get_attribute_dict() for vertex in self.vertices}

    def order_edges_by_hash_codes(self) -> Dict[str, Edge]:
        edges = {}
        for edge in self.edges:
            edge_data = {
                "edge_label": edge.label,
                "from_vertex_hash": self.get_vertex_hash_by_index(vertex_index=edge.origin),
                "to_vertex_hash": self.get_vertex_hash_by_index(vertex_index=edge.dest),
            }
            edge_hash = calculate_hash(edge_data)
            edges[edge_hash] = edge
        return edges

    def get_vertex_hash_by_index(self, vertex_index: int) -> str:
        return self.vertex_hash_cache.setdefault(vertex_index, self.vertices[vertex_index].get_hash())

    def update_vertex_attribute(
        self,
        vertex_index: int,
        attribute_key: str,
        attribute_value: Any,
        change_origin_id: int | None,
        attribute_at_dest: Optional[Union[str, List[str]]],
        transform_step: bool = False,
    ) -> None:
        if change_origin_id is None:
            # no need to proceed further
            return

        previous_breadcrumbs = []
        attribute_at_dest = self.vertices[change_origin_id].find_attribute(attribute_at_dest)
        if attribute_at_dest:
            previous_breadcrumbs = self.vertices[change_origin_id].changed_attributes.get(attribute_at_dest, [])
        self.vertices[vertex_index].update_attribute(
            attribute_key, attribute_value, change_origin_id, previous_breadcrumbs, attribute_at_dest
        )

    def update_vertices_configs(self) -> None:
        for vertex in self.vertices:
            changed_attributes = list(vertex.changed_attributes.keys())
            changed_attributes = filter_sub_keys(changed_attributes)
            self.update_vertex_config(vertex, changed_attributes)

    @staticmethod
    def update_vertex_config(vertex: TerraformBlock, changed_attributes: Union[List[str], Dict[str, Any]], dynamic_blocks: bool = False) -> None:
        if not changed_attributes:
            # skip, if there is no change
            return

        vertex_name = vertex.name
        updated_config = pickle_deepcopy(vertex.config)
        if vertex.block_type == BlockType.PROVIDER:
            # provider blocks set the alias as a suffix to the name, ex. name: "aws.prod"
            vertex_name = vertex_name.split(".")[0]
        if vertex.block_type != BlockType.LOCALS:
            parts = vertex_name.split(".")
            start = 0
            end = 1
            while end <= len(parts):
                cur_key = ".".join(parts[start:end])
                if cur_key in updated_config:
                    updated_config = updated_config[cur_key]
                    start = end
                end += 1

        for changed_attribute in changed_attributes:
            new_value = vertex.attributes.get(changed_attribute, None)
            if new_value is not None:
                if vertex.block_type == BlockType.LOCALS:
                    changed_attribute = changed_attribute.replace(f"{vertex_name}.", "")
                updated_config = update_dictionary_attribute(updated_config, changed_attribute, new_value, dynamic_blocks)

        if len(changed_attributes) > 0:
            if vertex.block_type == BlockType.LOCALS:
                updated_local_config = updated_config.get(vertex_name)
                update_dictionary_attribute(vertex.config, vertex_name, updated_local_config, dynamic_blocks)
                return

            update_dictionary_attribute(vertex.config, vertex_name, updated_config, dynamic_blocks)

    def get_resources_types_in_graph(self) -> List[str]:
        return self.module.get_resources_types()

    def _update_vertices_breadcrumbs_and_module_connections(self) -> None:
        """
        The function processes each vertex's breadcrumbs:
        1. Get more data to each vertex in breadcrumb (name, path, hash and type)
        2. If a breadcrumb is originated in a different module, it will have 'module_connection'=True
        3. If a vertex has a 'source module' we will add a special breadcrumb for it
        """
        for vertex in self.vertices:
            for attribute_key, breadcrumbs_list in vertex.changed_attributes.items():
                hash_breadcrumbs = []
                for breadcrumb in breadcrumbs_list:
                    v = self.vertices[breadcrumb.vertex_id]
                    breadcrumb = v.get_export_data()
                    breadcrumb["module_connection"] = self._determine_if_module_connection(breadcrumbs_list, v)
                    hash_breadcrumbs.append(breadcrumb)
                vertex.breadcrumbs[attribute_key] = hash_breadcrumbs
            if len(vertex.source_module) == 1:
                v = vertex
                source_module_data = []
                while len(v.source_module) == 1:
                    idx = list(v.source_module)[0]
                    v = self.vertices[idx]
                    module_data = v.get_export_data()
                    module_data["idx"] = idx
                    if hasattr(vertex, "source_module_object"):
                        module_data["source_module_object"] = v.source_module_object
                    source_module_data.append(module_data)
                source_module_data.reverse()
                vertex.breadcrumbs[CustomAttributes.SOURCE_MODULE] = source_module_data

    @staticmethod
    def _determine_if_module_connection(breadcrumbs_list: List[int], vertex_in_breadcrumbs: TerraformBlock) -> bool:
        """
        :param breadcrumbs_list: list of vertex's breadcrumbs
        :param vertex_in_breadcrumbs: one of the vertices in the breadcrumb list
        :return: True if vertex_in_breadcrumbs has in its module_connections at least one of the vertices in breadcrumbs_list
        """
        if not vertex_in_breadcrumbs.module_connections:
            return False
        for connection_list in vertex_in_breadcrumbs.module_connections.values():
            if any(i in breadcrumbs_list for i in connection_list):
                return True
        return False

    def get_dirname(self, path: str) -> str:
        dir_name = self.dirname_cache.get(path)
        if not dir_name:
            dir_name = os.path.dirname(path)
            self.dirname_cache[path] = dir_name
        return dir_name

    def get_abspath(self, path: str) -> str:
        dir_name = self.abspath_cache.get(path)
        if not dir_name:
            dir_name = os.path.abspath(path)
            self.abspath_cache[path] = dir_name
        return dir_name

    def _update_nested_modules_address(self) -> None:
        for vertex in self.vertices:
            if vertex.attributes.get(CustomAttributes.TF_RESOURCE_ADDRESS) is not None:
                # Can happen for example in `tf_plan` files as the address already exists
                continue
            if vertex.block_type not in parser_registry.context_parsers:
                continue
            source_module = vertex.breadcrumbs.get(CustomAttributes.SOURCE_MODULE)

            address_prefix = ''
            if source_module:
                for module in source_module:
                    address_prefix += f"{module.get('type')}.{module.get('name')}."

            context_parser = parser_registry.context_parsers[vertex.block_type]
            entity_context_path = context_parser.get_entity_context_path(vertex.config)
            resource_id = '.'.join(entity_context_path) if entity_context_path else vertex.name
            address = f'{address_prefix}{resource_id}'
            vertex.attributes[CustomAttributes.TF_RESOURCE_ADDRESS] = address

            vertex_context = vertex.config
            definition_path = context_parser.get_entity_definition_path(vertex.config)
            for path in definition_path:
                vertex_context = vertex_context.get(path, vertex_context)
            vertex_context[CustomAttributes.TF_RESOURCE_ADDRESS] = address

    def _should_add_edge(self, vertex: TerraformBlock, dest_module_path: str, module_node: TerraformBlock) -> bool:
        if not vertex.source_module_object:
            return False

        return (self.get_dirname(vertex.path) == dest_module_path) and \
            (
                vertex.source_module_object == module_node.source_module_object  # The vertex is in the same file
                or self.get_abspath(vertex.source_module_object.path)
                == self.get_abspath(module_node.path)  # The vertex is in the correct dependency path)
        )


def to_list(data: Any) -> list[Any] | dict[str, Any]:
    if isinstance(data, list) and len(data) == 1 and (isinstance(data[0], str) or isinstance(data[0], int)):
        return data
    elif isinstance(data, list):
        return [to_list(x) for x in data]
    elif isinstance(data, dict):
        return {key: to_list(val) for key, val in data.items()}
    else:
        return [data]


@overload
def update_dictionary_attribute(
        config: dict[str, Any], key_to_update: str, new_value: Any, dynamic_blocks: bool = False
) -> dict[str, Any]:
    ...


@overload
def update_dictionary_attribute(
        config: list[Any], key_to_update: str, new_value: Any, dynamic_blocks: bool = False
) -> list[Any]:
    ...


def update_dictionary_attribute(
        config: Union[List[Any], Dict[str, Any]], key_to_update: str, new_value: Any, dynamic_blocks: bool = False
) -> Union[List[Any], Dict[str, Any]]:
    key_parts = key_to_update.split(".")

    if isinstance(config, dict) and isinstance(key_parts, list):
        key = key_parts[0]
        inner_config = config.get(key)

        if inner_config is not None:
            if len(key_parts) == 1:
                if isinstance(inner_config, list) and not isinstance(new_value, list):
                    new_value = [new_value]
                config[key] = to_list(new_value) if dynamic_blocks else new_value
                return config
            else:
                config[key] = update_dictionary_attribute(inner_config, ".".join(key_parts[1:]), new_value, dynamic_blocks=dynamic_blocks)
        else:
            for key in config:
                config[key] = update_dictionary_attribute(config[key], key_to_update, new_value, dynamic_blocks=dynamic_blocks)
    if isinstance(config, list):
        return update_list_attribute(
            config=config,
            key_parts=key_parts,
            key_to_update=key_to_update,
            new_value=new_value,
            dynamic_blocks=dynamic_blocks
        )
    return config


def update_list_attribute(
    config: list[Any], key_parts: list[str], key_to_update: str, new_value: Any, dynamic_blocks: bool = False
) -> list[Any] | dict[str, Any]:
    """Updates a list attribute in the given config"""

    if not config:
        # happens when we can't correctly evaluate something, because of strange defaults or 'for_each' blocks
        return config

    if len(key_parts) == 1:
        idx = force_int(key_parts[0])
        inner_config = config[0]

        if idx is not None and isinstance(inner_config, list):
            if not inner_config:
                # happens when config = [[]]
                return config

            inner_config[idx] = new_value
            return config
    entry_to_update = int(key_parts[0]) if key_parts[0].isnumeric() else -1
    for i, config_value in enumerate(config):
        if entry_to_update == -1:
            config[i] = update_dictionary_attribute(config=config_value, key_to_update=key_to_update, new_value=new_value, dynamic_blocks=dynamic_blocks)
        elif entry_to_update == i:
            config[i] = update_dictionary_attribute(config=config_value, key_to_update=".".join(key_parts[1:]), new_value=new_value, dynamic_blocks=dynamic_blocks)

    return config


def get_vertex_as_tf_module(block: TerraformBlock) -> TFModule:
    return TFModule(block.path, block.name, block.source_module_object)



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class EKSPlatformVersion(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        NIST.800-53.r5 CA-9(1), NIST.800-53.r5 CM-2, NIST.800-53.r5 SI-2, NIST.800-53.r5 SI-2(2),
        NIST.800-53.r5 SI-2(4), NIST.800-53.r5 SI-2(5)
        EKS clusters should run on a supported Kubernetes version
        """
        name = "Ensure EKS clusters run on a supported Kubernetes version"
        id = "CKV_AWS_339"
        supported_resources = ("aws_eks_cluster",)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self) -> str:
        return "version"

    def get_expected_values(self) -> list[Any]:
        return ["1.23", "1.24", "1.25", "1.26", "1.27"]


check = EKSPlatformVersion()



from __future__ import annotations

from typing import Any

from checkov.common.images.workflow.image_referencer_manager import WorkflowImageReferencerManager
from checkov.gitlab_ci.image_referencer.provider import GitlabCiProvider


class GitlabCiImageReferencerManager(WorkflowImageReferencerManager):

    def __init__(self, workflow_config: dict[str, Any], file_path: str):
        provider = GitlabCiProvider(workflow_config=workflow_config, file_path=file_path)
        super().__init__(workflow_config, file_path, provider)



from __future__ import annotations

from checkov.common.images.workflow.image_referencer_provider import WorkflowImageReferencerProvider

from typing import Any, TYPE_CHECKING

if TYPE_CHECKING:
    from checkov.common.images.image_referencer import Image


class WorkflowImageReferencerManager:
    __slots__ = ("workflow_config", "file_path", "provider")

    def __init__(self, workflow_config: dict[str, Any], file_path: str, provider: WorkflowImageReferencerProvider):
        self.workflow_config = workflow_config
        self.file_path = file_path
        self.provider = provider

    def extract_images_from_workflow(self) -> list[Image]:
        images: list[Image] = self.provider.extract_images_from_workflow()
        return images



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AKSMaxPodsMinimum(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Azure Kubernetes Cluster (AKS) nodes should use a minimum number of 50 pods."
        id = "CKV_AZURE_168"
        supported_resources = ("azurerm_kubernetes_cluster", "azurerm_kubernetes_cluster_node_pool")
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        max_pods = 30  # default is 30

        self.evaluated_keys = ["max_pods"]
        if "max_pods" in conf.keys() and isinstance(conf["max_pods"][0], int):
            max_pods = conf["max_pods"][0]

        if "default_node_pool" in conf.keys():
            self.evaluated_keys = ["default_node_pool/max_pods"]
            pool = conf["default_node_pool"][0]
            if "max_pods" in pool.keys():
                max_pods_list = pool["max_pods"]
                if max_pods_list and isinstance(max_pods_list, list) and isinstance(max_pods_list[0], int):
                    max_pods = max_pods_list[0]

        if max_pods < 50:
            return CheckResult.FAILED

        return CheckResult.PASSED


check = AKSMaxPodsMinimum()



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class PolicyNoDSRI(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure DSRI is not enabled within security policies"
        id = "CKV_PAN_4"
        supported_resources = ('panos_security_policy', 'panos_security_rule_group')
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        # Check there is a rule defined in the resource
        if 'rule' in conf:

            # Report the area of evaluation
            self.evaluated_keys = ['rule']

            # Get all the rules defined in the resource
            rules = conf['rule']

            # Iterate over each rule
            for secrule in rules:

                # Check if DSRI is defined in the resource
                if 'disable_server_response_inspection' in secrule:

                    # If DSRI is defined, get the value
                    dsriflag = secrule['disable_server_response_inspection']

                    # Setting DSRI to true is a fail as server-to-client inspection will be disabled
                    if dsriflag[0]:
                        return CheckResult.FAILED

            # The other value for DSRI is false, which is a pass
            # Also, if the DSRI attribute is not explicitly set, the default is false, which is also a pass
            return CheckResult.PASSED

        # If there's no rules we have nothing to check
        return CheckResult.UNKNOWN


check = PolicyNoDSRI()



from __future__ import annotations

import json
import os
from typing import List, Dict, Any, Set, Callable, Tuple, TYPE_CHECKING, cast

from checkov.common.typing import TFDefinitionKeyType
from checkov.common.util.data_structures_utils import pickle_deepcopy
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.graph_components.blocks import TerraformBlock
from checkov.terraform.parser_functions import handle_dynamic_values
from hcl2 import START_LINE, END_LINE

if TYPE_CHECKING:
    from typing_extensions import TypeAlias

_AddBlockTypeCallable: TypeAlias = "Callable[[Module, list[dict[str, dict[str, Any]]], TFDefinitionKeyType], None]"


class Module:
    def __init__(
            self,
            source_dir: str,
            external_modules_source_map: Dict[Tuple[str, str], str],
    ) -> None:
        # when adding a new field be sure to add it to the equality function below
        self.external_modules_source_map = external_modules_source_map
        self.path = ""
        self.blocks: List[TerraformBlock] = []
        self.customer_name = ""
        self.account_id = ""
        self.source = ""
        self.resources_types: Set[str] = set()
        self.source_dir = source_dir
        self.render_dynamic_blocks_env_var = os.getenv('CHECKOV_RENDER_DYNAMIC_MODULES', 'True')

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, Module):
            return False

        return self.external_modules_source_map == other.external_modules_source_map and \
            self.path == other.path and \
            self.customer_name == other.customer_name and \
            self.account_id == other.account_id and \
            self.source == other.source and \
            self.resources_types == other.resources_types and \
            self.source_dir == other.source_dir and \
            self.blocks == other.blocks

    def to_dict(self) -> dict[str, Any]:
        return {
            'external_modules_source_map': self.external_modules_source_map,
            'path': self.path,
            'customer_name': self.customer_name,
            'account_id': self.account_id,
            'source': self.source,
            'resources_types': self.resources_types,
            'source_dir': self.source_dir,
            'render_dynamic_blocks_env_var': self.render_dynamic_blocks_env_var,
            'blocks': [block.to_dict() for block in self.blocks]
        }

    @staticmethod
    def from_dict(module_dict: dict[str, Any]) -> Module:
        module = Module(source_dir=module_dict.get('source_dir', ''),
                        external_modules_source_map=module_dict.get('external_modules_source_map', {})
                        )
        module.blocks = [TerraformBlock.from_dict(block) for block in module_dict.get('blocks', [])]
        module.path = module_dict.get('path', '')
        module.customer_name = module_dict.get('customer_name', '')
        module.account_id = module_dict.get('account_id', '')
        module.source = module_dict.get('source', '')
        module.resources_types = module_dict.get('resources_types', set())
        module.source_dir = module_dict.get('source_dir', '')
        module.render_dynamic_blocks_env_var = module_dict.get('render_dynamic_blocks_env_var', '')
        return module

    def add_blocks(
            self, block_type: str, blocks: List[Dict[str, Dict[str, Any]]], path: TFDefinitionKeyType, source: str
    ) -> None:
        self.source = source
        if block_type in self._block_type_to_func:
            self._block_type_to_func[block_type](self, blocks, path)

    def _add_to_blocks(self, block: TerraformBlock) -> None:
        if isinstance(block.path, str):
            block.source_module_object = None
            block.path = block.path
        else:
            block.source_module_object = block.path.tf_source_modules
            block.path = block.path.file_path
        self.blocks.append(block)
        return

    def _add_provider(self, blocks: List[Dict[str, Dict[str, Any]]], path: TFDefinitionKeyType) -> None:
        for provider_dict in blocks:
            for name in provider_dict:
                attributes = provider_dict[name]
                if START_LINE not in attributes or END_LINE not in attributes:
                    return
                provider_name = name
                if isinstance(attributes, dict):
                    alias = attributes.get("alias")
                    if alias:
                        provider_name = f"{provider_name}.{alias[0]}"
                provider_block = TerraformBlock(
                    block_type=BlockType.PROVIDER,
                    name=provider_name,
                    config=provider_dict,
                    path=path,
                    attributes=attributes,
                    source=self.source,
                )
                self._add_to_blocks(provider_block)

    def _add_variable(self, blocks: List[Dict[str, Dict[str, Any]]], path: TFDefinitionKeyType) -> None:
        for variable_dict in blocks:
            for name in variable_dict:
                attributes = variable_dict[name]
                variable_block = TerraformBlock(
                    block_type=BlockType.VARIABLE,
                    name=name,
                    config=variable_dict,
                    path=path,
                    attributes=attributes,
                    source=self.source,
                )
                self._add_to_blocks(variable_block)

    def _add_locals(self, blocks: List[Dict[str, Dict[str, Any]]], path: TFDefinitionKeyType) -> None:
        for blocks_section in blocks:
            for name in blocks_section:
                if name in (START_LINE, END_LINE):
                    # locals block generates single block sections for the start/end lines
                    continue

                local_block = TerraformBlock(
                    block_type=BlockType.LOCALS,
                    name=name,
                    config={name: blocks_section[name]},
                    path=path,
                    attributes={name: blocks_section[name]},
                    source=self.source,
                )
                self._add_to_blocks(local_block)

    def _add_output(self, blocks: List[Dict[str, Dict[str, Any]]], path: TFDefinitionKeyType) -> None:
        for output_dict in blocks:
            for name, attributes in output_dict.items():
                if isinstance(attributes, dict):
                    output_block = TerraformBlock(
                        block_type=BlockType.OUTPUT,
                        name=name,
                        config=output_dict,
                        path=path,
                        attributes={"value": attributes.get("value")},
                        source=self.source,
                    )
                    self._add_to_blocks(output_block)

    def _add_module(self, blocks: List[Dict[str, Dict[str, Any]]], path: TFDefinitionKeyType) -> None:
        for module_dict in blocks:
            for name, attributes in module_dict.items():
                if isinstance(attributes, dict):
                    module_block = TerraformBlock(
                        block_type=BlockType.MODULE,
                        name=name,
                        config=module_dict,
                        path=path,
                        attributes=attributes,
                        source=self.source,
                    )
                    self._add_to_blocks(module_block)

    def _add_resource(self, blocks: List[Dict[str, Dict[str, Any]]], path: TFDefinitionKeyType) -> None:
        for resource_dict in blocks:
            for resource_type, resources in resource_dict.items():
                self.resources_types.add(resource_type)
                for name, resource_conf in resources.items():
                    attributes = self.clean_bad_characters(resource_conf)
                    dynamic_attributes = None
                    if not isinstance(attributes, dict):
                        continue
                    if self.render_dynamic_blocks_env_var.lower() == 'false':
                        has_dynamic_block = False
                    else:
                        old_attributes = pickle_deepcopy(attributes)
                        has_dynamic_block = handle_dynamic_values(attributes)
                        dynamic_attributes = {k: attributes[k] for k in set(attributes) - set(old_attributes)}
                    provisioner = attributes.get("provisioner")
                    if provisioner:
                        self._handle_provisioner(provisioner, attributes)
                    attributes["resource_type"] = [resource_type]
                    block_name = f"{resource_type}.{name}"
                    resource_block = TerraformBlock(
                        block_type=BlockType.RESOURCE,
                        name=block_name,
                        config=self.clean_bad_characters(resource_dict),
                        path=path,
                        attributes=attributes,
                        id=block_name,
                        source=self.source,
                        has_dynamic_block=has_dynamic_block,
                        dynamic_attributes=dynamic_attributes
                    )
                    self._add_to_blocks(resource_block)

    @staticmethod
    def clean_bad_characters(resource_conf: dict[str, Any]) -> dict[str, Any]:
        try:
            return cast("dict[str, Any]", json.loads(json.dumps(resource_conf).replace("\\\\", "\\")))
        except json.JSONDecodeError:
            return resource_conf

    def _add_data(self, blocks: List[Dict[str, Dict[str, Any]]], path: TFDefinitionKeyType) -> None:
        for data_dict in blocks:
            for data_type in data_dict:
                for name in data_dict[data_type]:
                    block_name = f"{data_type}.{name}"
                    data_block = TerraformBlock(
                        block_type=BlockType.DATA,
                        name=block_name,
                        config=data_dict,
                        path=path,
                        attributes=data_dict.get(data_type, {}).get(name, {}),
                        id=block_name,
                        source=self.source,
                    )
                    self._add_to_blocks(data_block)

    def _add_terraform_block(self, blocks: List[Dict[str, Dict[str, Any]]], path: TFDefinitionKeyType) -> None:
        for terraform_dict in blocks:
            terraform_block = TerraformBlock(
                block_type=BlockType.TERRAFORM,
                name="",
                config=terraform_dict,
                path=path,
                attributes=terraform_dict,
                source=self.source,
            )
            self._add_to_blocks(terraform_block)

    def _add_tf_var(self, blocks: list[Dict[str, Dict[str, Any]]], path: TFDefinitionKeyType) -> None:
        for block in blocks:
            for tf_var_name, attributes in block.items():
                tfvar_block = TerraformBlock(
                    block_type=BlockType.TF_VARIABLE,
                    name=tf_var_name,
                    config={tf_var_name: attributes},
                    path=path,
                    attributes=attributes,
                    source=self.source,
                )
                self._add_to_blocks(tfvar_block)

    @staticmethod
    def _handle_provisioner(provisioner: List[Dict[str, Any]], attributes: Dict[str, Any]) -> None:
        for pro in provisioner:
            if pro.get("local-exec"):
                inner_attributes = TerraformBlock.get_inner_attributes("provisioner/local-exec", pro["local-exec"])
                attributes.update(inner_attributes)
            elif pro.get("remote-exec"):
                inner_attributes = TerraformBlock.get_inner_attributes("provisioner/remote-exec", pro["remote-exec"])
                attributes.update(inner_attributes)
        del attributes["provisioner"]

    def get_resources_types(self) -> List[str]:
        return list(self.resources_types)

    _block_type_to_func: Dict[str, _AddBlockTypeCallable] = {  # noqa: CCE003  # a static attribute
        BlockType.DATA: _add_data,
        BlockType.LOCALS: _add_locals,
        BlockType.MODULE: _add_module,
        BlockType.OUTPUT: _add_output,
        BlockType.PROVIDER: _add_provider,
        BlockType.RESOURCE: _add_resource,
        BlockType.TERRAFORM: _add_terraform_block,
        BlockType.TF_VARIABLE: _add_tf_var,
        BlockType.VARIABLE: _add_variable,
    }



from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck

BASIC_ROLES = {
    "roles/owner",
    "roles/editor",
    "roles/viewer",
}


class AbsGoogleBasicRoles(BaseResourceCheck):
    def scan_resource_conf(self, conf):
        self.evaluated_keys = ['role']
        role = conf.get("role")
        if role and isinstance(role, list) and role[0] in BASIC_ROLES:
            return CheckResult.FAILED
        return CheckResult.PASSED



from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AmazonMQBrokerPublicAccess(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure Amazon MQ Broker should not have public access"
        id = "CKV_AWS_69"
        supported_resources = ('AWS::AmazonMQ::Broker',)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            missing_block_result=CheckResult.FAILED,
        )

    def get_expected_value(self) -> Any:
        return False

    def get_inspected_key(self) -> str:
        """
            validates Amazon MQ Broker should not have public access
            https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-amazonmq-broker.html
        :return: <CheckResult>
        """
        return 'Properties/PubliclyAccessible'


check = AmazonMQBrokerPublicAccess()



from __future__ import annotations

import logging
from typing import Any, Dict, List, Optional
import yaml

from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.util.file_utils import decompress_file_gzip_base64


def load_detectors() -> list[dict[str, Any]]:
    detectors: List[dict[str, Any]] = []
    try:
        customer_run_config_response = bc_integration.customer_run_config_response
        policies_list: List[dict[str, Any]] = []
        if customer_run_config_response:
            policies_list = customer_run_config_response.get('secretsPolicies', [])
    except Exception as e:
        logging.error(f"Failed to get detectors from customer_run_config_response, error: {e}")
        return []

    if policies_list:
        detectors = modify_secrets_policy_to_detectors(policies_list)
    if detectors:
        logging.info(f"Successfully loaded {len(detectors)} detectors from bc_integration")
    return detectors


def modify_secrets_policy_to_detectors(policies_list: List[dict[str, Any]]) -> List[dict[str, Any]]:
    secrets_list = transforms_policies_to_detectors_list(policies_list)
    logging.debug(f"(modify_secrets_policy_to_detectors) len secrets_list = {len(secrets_list)}")
    return secrets_list


def add_to_custom_detectors(custom_detectors: List[Dict[str, Any]], name: str, check_id: str, regex: str,
                            is_custom: str, is_multiline: bool = False, supported_files: Optional[List[str]] = None) -> None:
    custom_detectors.append({
        'Name': name,
        'Check_ID': check_id,
        'Regex': regex,
        'isCustom': is_custom,
        'isMultiline': is_multiline,
        'supportedFiles': supported_files if supported_files else []
    })


def add_detectors_from_condition_query(custom_detectors: List[Dict[str, Any]], condition_query: Dict[str, Any],
                                       secret_policy: Dict[str, Any], check_id: str) -> bool:
    parsed = False
    cond_type = condition_query['cond_type']
    if cond_type == 'secrets':
        value = condition_query['value']
        if type(value) is str:
            value = [value]
        for regex in value:
            parsed = True
            add_to_custom_detectors(custom_detectors, secret_policy['title'], check_id, regex,
                                    secret_policy['isCustom'])
    return parsed


def add_detectors_from_code(custom_detectors: List[Dict[str, Any]], code: str, secret_policy: Dict[str, Any],
                            check_id: str) -> bool:
    parsed = False
    code_dict = yaml.safe_load(code)
    if 'definition' in code_dict:
        if 'value' in code_dict['definition'] and 'is_runnable' not in code_dict['definition']:
            parsed = True
            if type(code_dict['definition']['value']) is str:
                code_dict['definition']['value'] = [code_dict['definition']['value']]
            for regex in code_dict['definition']['value']:
                add_to_custom_detectors(
                    custom_detectors,
                    secret_policy['title'],
                    check_id,
                    regex,
                    secret_policy['isCustom'],
                    code_dict['definition'].get("multiline", False),
                    code_dict['definition'].get("supported_files", [])
                )
    return parsed


def transforms_policies_to_detectors_list(custom_secrets: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
    custom_detectors: List[Dict[str, Any]] = []
    condition_query = None
    for secret_policy in custom_secrets:
        parsed = False
        check_id = secret_policy['checkovCheckId'] if secret_policy['checkovCheckId'] else \
            secret_policy['incidentId']
        code = secret_policy['code']
        if 'conditionQuery' in secret_policy:
            condition_query = secret_policy['conditionQuery']
        if condition_query:
            parsed = add_detectors_from_condition_query(custom_detectors, condition_query, secret_policy, check_id)
        elif code:
            parsed = add_detectors_from_code(custom_detectors, code, secret_policy, check_id)
        if not parsed:
            logging.info(f"policy : {secret_policy} could not be parsed")
    return custom_detectors


def get_runnable_plugins(policies: List[Dict[str, Any]]) -> Dict[str, str]:
    runnables: dict[str, str] = {}
    for policy in policies:
        code = policy['code']
        if code:
            try:
                code_dict = yaml.safe_load(code)
                if 'definition' in code_dict:
                    if 'is_runnable' in code_dict['definition'] and 'value' in code_dict['definition']:
                        encoded_payload = code_dict['definition']['value']
                        if isinstance(encoded_payload, list):
                            encoded_payload = encoded_payload[0]
                        decoded_payload = decompress_file_gzip_base64(encoded_payload)
                        name: str = policy['title']
                        runnables[name] = decoded_payload.decode('utf8')
            except Exception as e:
                logging.warning(f"Could not parse runnable policy {policy['title']} due to: {e}")
    return runnables



from __future__ import annotations
from typing import Any, Mapping, TYPE_CHECKING, Union

from checkov.common.graph.graph_builder import CustomAttributes
from checkov.kubernetes.image_referencer.base_provider import BaseKubernetesProvider
from checkov.common.images.graph.image_referencer_provider import _ExtractImagesCallableAlias

if TYPE_CHECKING:
    from igraph import Graph
    from networkx import DiGraph


class BaseHelmProvider(BaseKubernetesProvider):
    def __init__(self, graph_connector: Union[Graph, DiGraph],
                 supported_resource_types: dict[str, _ExtractImagesCallableAlias] | Mapping[str, _ExtractImagesCallableAlias],
                 original_root_dir: str, temp_root_dir: str) -> None:
        super().__init__(
            graph_connector=graph_connector,
            supported_resource_types=supported_resource_types,
        )
        self.original_root_dir = original_root_dir
        self.temp_root_dir = temp_root_dir

    def _get_resource_path(self, resource: dict[str, Any]) -> str:
        k8s_path = resource.get(CustomAttributes.FILE_PATH, "")
        return str(k8s_path.replace(self.temp_root_dir, self.original_root_dir, 1))




from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class LBListenerUsesSecureProtocols(BaseResourceCheck):
    def __init__(self):
        name = "Ensure LB Listener uses only secure protocols"
        id = "CKV_NCP_13"
        supported_resources = ('ncloud_lb_listener',)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'protocol' in conf.keys():
            protocol = conf['protocol'][0]
            if protocol in ('HTTPS', 'TLS'):
                if 'tls_min_version_type' in conf.keys():
                    if conf['tls_min_version_type'] == ['TLSV12']:
                        return CheckResult.PASSED
            return CheckResult.FAILED


check = LBListenerUsesSecureProtocols()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from typing import List, Any


class AppConfigLocalAuth(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        """
        Every request to an Azure App Configuration resource must be authenticated.
        By default, requests can be authenticated with either Azure Active Directory (Azure AD) credentials,
        or by using an access key. Of these two types of authentication schemes,
        Azure AD provides superior security and ease of use over access keys, and is recommended by Microsoft.
        """
        name = "Ensure 'local_auth_enabled' is set to 'False'"
        id = "CKV_AZURE_184"
        supported_resources = ("azurerm_app_configuration",)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_attribute_result=CheckResult.FAILED)

    def get_inspected_key(self) -> str:
        return "local_auth_enabled"

    def get_forbidden_values(self) -> List[Any]:
        return [True]


check = AppConfigLocalAuth()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class StorageBlobServiceContainerPrivateAccess(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that 'Public access level' is set to Private for blob containers"
        id = "CKV_AZURE_34"
        supported_resources = ['azurerm_storage_container']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self):
        return 'container_access_type/[0]'

    def get_expected_value(self):
        return 'private'


check = StorageBlobServiceContainerPrivateAccess()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE


class FileSystemEncryption(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure OCI File System is Encrypted with a customer Managed Key"
        id = "CKV_OCI_15"
        supported_resources = ['oci_file_storage_file_system']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'kms_key_id'

    def get_expected_value(self):
        return ANY_VALUE


check = FileSystemEncryption()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class AppServiceSkuMinimum(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        # "Azure App Services provide a range of different plans that can be used to scale your application.
        # Each plan provides different levels of performance and features.
        # To get you started a number of entry level plans are available.
        # The Free, Shared, and Basic plans can be used for limited testing and development.
        # These plans are not suitable for production use.
        # Production workloads are best suited to standard and premium plans with PremiumV3 the newest plan."
        name = "Ensure App Service plan suitable for production use"
        id = "CKV_AZURE_211"
        supported_resources = ['azurerm_service_plan']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'sku_name'

    def get_forbidden_values(self) -> list[Any]:
        return ["B1", "B2", "B3", "F1", "D1"]


check = AppServiceSkuMinimum()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories
from typing import Any
from checkov.common.models.consts import ANY_VALUE


class NetworkFirewallUsesCMK(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure that Network firewall encryption is via a CMK"
        id = "CKV_AWS_345"
        supported_resources = ['aws_networkfirewall_firewall', 'aws_networkfirewall_rule_group']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'encryption_configuration/[0]/key_id'

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = NetworkFirewallUsesCMK()



from checkov.terraform.checks.resource.ncp.AccessControlGroupInboundRule import AccessControlGroupInboundRule


class AccessControlGroupRuleInboundPort80(AccessControlGroupInboundRule):
    def __init__(self):
        super().__init__(check_id="CKV_NCP_25", port=80)


check = AccessControlGroupRuleInboundPort80()



from __future__ import annotations

import os
import logging
from pathlib import Path
from typing import cast, Type, TYPE_CHECKING, Any

from typing_extensions import TypeAlias  # noqa[TC002]

from checkov.bicep.graph_builder.context_definitions import build_definitions_context
from checkov.bicep.checks.param.registry import registry as param_registry
from checkov.bicep.checks.resource.registry import registry as resource_registry
from checkov.bicep.graph_builder.graph_to_tf_definitions import convert_graph_vertices_to_tf_definitions
from checkov.bicep.graph_builder.local_graph import BicepLocalGraph
from checkov.bicep.graph_manager import BicepGraphManager
from checkov.bicep.image_referencer.manager import BicepImageReferencerManager
from checkov.bicep.parser import Parser
from checkov.bicep.utils import clean_file_path, get_scannable_file_paths
from checkov.common.checks_infra.registry import get_graph_checks_registry

from checkov.common.typing import LibraryGraphConnector
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.graph.graph_builder.consts import GraphSource
from checkov.common.images.image_referencer import ImageReferencerMixin
from checkov.common.output.extra_resource import ExtraResource
from checkov.common.output.graph_record import GraphRecord
from checkov.common.output.record import Record
from checkov.common.output.report import Report
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.runners.base_runner import BaseRunner, CHECKOV_CREATE_GRAPH
from checkov.common.typing import _CheckResult
from checkov.common.util.secrets import omit_secret_value_from_checks
from checkov.common.util.suppression import collect_suppressions_for_report
from checkov.runner_filter import RunnerFilter

if TYPE_CHECKING:
    from checkov.common.checks.base_check_registry import BaseCheckRegistry
    from checkov.common.checks_infra.registry import Registry
    from checkov.common.graph.checks_infra.registry import BaseRegistry
    from checkov.common.images.image_referencer import Image
    from networkx import DiGraph
    from pycep.typing import BicepJson
    from typing_extensions import Literal

_BicepContext: TypeAlias = "dict[str, dict[str, Any]]"
_BicepDefinitions: TypeAlias = "dict[Path, BicepJson]"


class Runner(ImageReferencerMixin[None], BaseRunner[_BicepDefinitions, _BicepContext, BicepGraphManager]):
    check_type = CheckType.BICEP  # noqa: CCE003  # a static attribute

    block_type_registries: 'dict[Literal["parameters", "resources"], BaseCheckRegistry]' = {  # noqa: CCE003  # a static attribute
        "parameters": param_registry,
        "resources": resource_registry,
    }

    def __init__(
        self,
        db_connector: LibraryGraphConnector | None = None,
        source: str = GraphSource.BICEP,
        graph_class: Type[BicepLocalGraph] = BicepLocalGraph,
        graph_manager: BicepGraphManager | None = None,
        external_registries: list[BaseRegistry] | None = None
    ) -> None:
        super().__init__(file_extensions=['.bicep'])
        db_connector = db_connector or self.db_connector
        self.external_registries = external_registries if external_registries else []
        self.graph_class = graph_class
        self.graph_manager: BicepGraphManager = (
            graph_manager if graph_manager else BicepGraphManager(source=source, db_connector=db_connector)
        )
        self.graph_registry: Registry = get_graph_checks_registry(self.check_type)

        self.context: _BicepContext = {}
        self.definitions: _BicepDefinitions = {}
        self.definitions_raw: dict[Path, list[tuple[int, str]]] = {}    # type:ignore[assignment]
        self.root_folder: str | Path | None = None

    def run(
        self,
        root_folder: str | Path | None,
        external_checks_dir: list[str] | None = None,
        files: list[str] | None = None,
        runner_filter: RunnerFilter | None = None,
        collect_skip_comments: bool = True,
    ) -> Report | list[Report]:
        runner_filter = runner_filter or RunnerFilter()
        if not runner_filter.show_progress_bar:
            self.pbar.turn_off_progress_bar()

        report = Report(Runner.check_type)
        self.root_folder = root_folder

        if not self.context or not self.definitions:
            file_paths = get_scannable_file_paths(
                root_folder=root_folder, files=files, excluded_paths=runner_filter.excluded_paths
            )

            if not file_paths:
                return report

            self.definitions, self.definitions_raw, parsing_errors = Parser().get_files_definitions(file_paths)

            report.add_parsing_errors(parsing_errors)

            if external_checks_dir:
                for directory in external_checks_dir:
                    resource_registry.load_external_checks(directory)

                    if CHECKOV_CREATE_GRAPH:
                        self.graph_registry.load_external_checks(directory)

            self.context = build_definitions_context(definitions=self.definitions, definitions_raw=self.definitions_raw)

            if CHECKOV_CREATE_GRAPH:
                logging.info("Creating Bicep graph")
                local_graph = self.graph_manager.build_graph_from_definitions(self.definitions)
                logging.info("Successfully created Bicep graph")

                self.graph_manager.save_graph(local_graph)
                self.definitions, self.breadcrumbs = convert_graph_vertices_to_tf_definitions(
                    vertices=local_graph.vertices, root_folder=root_folder
                )

        self.pbar.initiate(len(self.definitions))

        # run Python checks
        self.add_python_check_results(report=report, runner_filter=runner_filter, root_folder=root_folder)

        # run graph checks
        if CHECKOV_CREATE_GRAPH:
            self.add_graph_check_results(report=report, runner_filter=runner_filter)

        if runner_filter.run_image_referencer:
            if files:
                # 'root_folder' shouldn't be empty to remove the whole path later and only leave the shortened form
                root_folder = os.path.split(os.path.commonprefix(files))[0]

            image_report = self.check_container_image_references(
                graph_connector=self.graph_manager.get_reader_endpoint(),
                root_path=root_folder,
                runner_filter=runner_filter,
            )

            if image_report:
                # due too many tests failing only return a list, if there is an image report
                return [report, image_report]

        return report

    def set_definitions_raw(self, definitions_raw: dict[Path, list[tuple[int, str]]]) -> None:
        self.definitions_raw = definitions_raw

    def add_python_check_results(
        self, report: Report, runner_filter: RunnerFilter, root_folder: str | Path | None
    ) -> None:
        """Adds Python check results to given report"""

        for file_path, definition in self.definitions.items():
            self.pbar.set_additional_data({'Current File Scanned': os.path.relpath(file_path, root_folder)})
            for block_type, registry in Runner.block_type_registries.items():
                block_type_confs = definition.get(block_type)
                if block_type_confs:
                    for name, conf in block_type_confs.items():
                        results = registry.scan(
                            scanned_file=str(file_path),
                            entity={name: conf},
                            skipped_checks=[],
                            runner_filter=runner_filter
                        )

                        if results:
                            file_code_lines = self.definitions_raw[file_path]
                            start_line = conf["__start_line__"]
                            end_line = conf["__end_line__"]

                            cleaned_path = clean_file_path(file_path)
                            resource_id = f"{conf['type']}.{name}"
                            report.add_resource(f"{cleaned_path}:{resource_id}")

                            suppressions = collect_suppressions_for_report(
                                code_lines=file_code_lines[start_line - 1 : end_line]
                            )

                            for check, check_result in results.items():
                                if check.id in suppressions.keys():
                                    check_result = suppressions[check.id]
                                elif check.bc_id and check.bc_id in suppressions.keys():
                                    check_result = suppressions[check.bc_id]

                                censored_code_lines = omit_secret_value_from_checks(
                                    check=check,
                                    check_result=check_result,
                                    entity_code_lines=file_code_lines[start_line - 1 : end_line],
                                    entity_config=conf,
                                    resource_attributes_to_omit=runner_filter.resource_attr_to_omit
                                )

                                record = Record(
                                    check_id=check.id,
                                    bc_check_id=check.bc_id,
                                    check_name=check.name,
                                    check_result=check_result,
                                    code_block=censored_code_lines,
                                    file_path=self.extract_file_path_from_abs_path(cleaned_path),
                                    file_line_range=[start_line, end_line],
                                    resource=resource_id,
                                    check_class=check.__class__.__module__,
                                    file_abs_path=str(file_path.absolute()),
                                    evaluations=None,
                                    severity=check.severity,
                                )
                                record.set_guideline(check.guideline)
                                report.add_record(record=record)
                        elif conf.get("existing") is False:
                            # resources without checks, but not existing ones

                            cleaned_path = clean_file_path(file_path)
                            resource_id = f"{conf['type']}.{name}"
                            report.extra_resources.add(
                                ExtraResource(
                                    file_abs_path=str(file_path.absolute()),
                                    file_path=self.extract_file_path_from_abs_path(cleaned_path),
                                    resource=resource_id,
                                )
                            )
            self.pbar.update()
        self.pbar.close()

    def extract_file_path_from_abs_path(self, path: Path) -> str:
        return f"/{os.path.relpath(path, self.root_folder)}"

    def add_graph_check_results(self, report: Report, runner_filter: RunnerFilter) -> None:
        """Adds YAML check results to given report"""

        checks_results = self.run_graph_checks_results(runner_filter, self.check_type)

        for check, check_results in checks_results.items():
            for check_result in check_results:
                entity = check_result["entity"]
                entity_file_path = Path(entity[CustomAttributes.FILE_PATH])

                clean_check_result: _CheckResult = {
                    "result": check_result["result"],
                    "evaluated_keys": check_result["evaluated_keys"],
                }

                file_code_lines = self.definitions_raw[entity_file_path]
                start_line = entity["__start_line__"]
                end_line = cast("int", entity["__end_line__"])

                record = Record(
                    check_id=check.id,
                    bc_check_id=check.bc_id,
                    check_name=check.name,
                    check_result=clean_check_result,
                    code_block=file_code_lines[start_line - 1 : end_line],
                    file_path=self.extract_file_path_from_abs_path(clean_file_path(entity_file_path)),
                    file_line_range=[start_line, end_line],
                    resource=entity[CustomAttributes.ID],
                    check_class=check.__class__.__module__,
                    file_abs_path=str(entity_file_path.absolute()),
                    evaluations=None,
                    severity=check.severity,
                )
                if self.breadcrumbs:
                    breadcrumb = self.breadcrumbs.get(record.file_path, {}).get(record.resource)
                    if breadcrumb:
                        record = GraphRecord(record, breadcrumb)
                record.set_guideline(check.guideline)
                report.add_record(record=record)

    def extract_images(
        self,
        graph_connector: DiGraph | None = None,
        definitions: None = None,
        definitions_raw: dict[str, list[tuple[int, str]]] | None = None,
    ) -> list[Image]:
        if not graph_connector:
            # should not happen
            return []

        manager = BicepImageReferencerManager(graph_connector=graph_connector)
        images = manager.extract_images_from_resources()

        return images



#
# Small customisations below to import resource registry and add the
# resource type to the supported_entities
from __future__ import annotations

from checkov.example_runner.checks.base_example_runner_check import BaseExampleRunnerCheck

# The base check required you to import the resource check registry
from checkov.example_runner.checks.job_registry import registry


class BaseExampleRunnerJobCheck(BaseExampleRunnerCheck):
    def __init__(self, name: str, id: str, block_type: str, path: str | None = None) -> None:
        super().__init__(
            name=name,
            id=id,
            # Set up your runner for the correct resource types
            # This is the same string is you defined in line 20
            # in your runner.py
            supported_entities=("jobs",),
            block_type=block_type,
        )
        self.path = path
        registry.register(self)



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck


class CosmosDBDisableAccessKeyWrite(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure cosmosdb does not allow privileged escalation by restricting management plane changes"
        id = "CKV_AZURE_132"
        supported_resources = ('Microsoft.DocumentDB/databaseAccounts',)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        if 'properties' in conf:
            if "disableKeyBasedMetadataWriteAccess" in conf['properties'] and conf['properties']['disableKeyBasedMetadataWriteAccess']:
                return CheckResult.PASSED
        return CheckResult.FAILED


check = CosmosDBDisableAccessKeyWrite()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from typing import Any, List


class RDSCACertIsRecent(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure RDS uses a modern CaCert"
        id = "CKV_AWS_211"
        supported_resources = ["aws_db_instance"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            missing_block_result=CheckResult.PASSED
        )

    def get_inspected_key(self) -> str:
        return "ca_cert_identifier"

    def get_expected_values(self) -> List[Any]:
        return ["rds-ca-rsa2048-g1", "rds-ca-rsa4096-g1", "rds-ca-ecc384-g1"]


check = RDSCACertIsRecent()



from __future__ import annotations

import re
from typing import Any

from checkov.common.models.enums import CheckResult
from checkov.github_actions.checks.base_github_action_check import BaseGithubActionsCheck
from checkov.github_actions.common.shell_injection_list import terms as bad_inputs
from checkov.yaml_doc.enums import BlockType


class DontAllowShellInjection(BaseGithubActionsCheck):
    def __init__(self) -> None:
        name = "Ensure run commands are not vulnerable to shell injection"
        id = "CKV_GHA_2"
        super().__init__(
            name=name,
            id=id,
            block_type=BlockType.ARRAY,
            supported_entities=('jobs', 'jobs.*.steps[]')
        )

    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        if not isinstance(conf, dict):
            return CheckResult.UNKNOWN, conf

        if "run" not in conf:
            return CheckResult.PASSED, conf
        run = conf.get("run", "")
        for term in bad_inputs:
            if re.search(term, run):
                return CheckResult.FAILED, conf

        return CheckResult.PASSED, conf


check = DontAllowShellInjection()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from typing import Any, List


class ECSServicePublicIP(BaseResourceNegativeValueCheck):
    def __init__(self):
        """
        NIST.800-53.r5 AC-21, NIST.800-53.r5 AC-3, NIST.800-53.r5 AC-3(7), NIST.800-53.r5 AC-4, NIST.800-53.r5 AC-4(21),
        NIST.800-53.r5 AC-6, NIST.800-53.r5 SC-7, NIST.800-53.r5 SC-7(11), NIST.800-53.r5 SC-7(16),
        NIST.800-53.r5 SC-7(20), NIST.800-53.r5 SC-7(21), NIST.800-53.r5 SC-7(3), NIST.800-53.r5 SC-7(4),
        NIST.800-53.r5 SC-7(9)
        ECS services should not have public IP addresses assigned to them automatically
        """
        name = "Ensure ECS services do not have public IP addresses assigned to them automatically"
        id = "CKV_AWS_333"
        supported_resources = ["aws_ecs_service"]
        categories = [CheckCategories.LOGGING]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def get_inspected_key(self) -> str:
        return "network_configuration/[0]/assign_public_ip"

    def get_forbidden_values(self) -> List[Any]:
        return [True]


check = ECSServicePublicIP()



from __future__ import annotations

from collections import OrderedDict
from pathlib import Path
from typing import TYPE_CHECKING

from dockerfile_parse import DockerfileParser
from dockerfile_parse.constants import COMMENT_INSTRUCTION

from checkov.common.typing import _SkippedCheck
from checkov.common.util.suppression import collect_suppressions_for_context

if TYPE_CHECKING:
    from dockerfile_parse.parser import _Instruction  # only in extra_stubs


def parse(filename: str | Path) -> tuple[dict[str, list[_Instruction]], list[str]]:
    with open(filename) as dockerfile:
        dfp = DockerfileParser(fileobj=dockerfile)
        return dfp_group_by_instructions(dfp)


def dfp_group_by_instructions(dfp: DockerfileParser) -> tuple[dict[str, list[_Instruction]], list[str]]:
    result: dict[str, list[_Instruction]] = OrderedDict()
    for instruction in dfp.structure:
        instruction_literal = instruction["instruction"]
        if instruction_literal not in result:
            result[instruction_literal] = []
        result[instruction_literal].append(instruction)
    return result, dfp.lines


def collect_skipped_checks(parse_result: dict[str, list[_Instruction]]) -> list[_SkippedCheck]:
    skipped_checks = []

    if COMMENT_INSTRUCTION in parse_result:
        # line number doesn't matter
        comment_lines = [(0, comment["value"]) for comment in parse_result[COMMENT_INSTRUCTION]]
        skipped_checks = collect_suppressions_for_context(code_lines=comment_lines)

    return skipped_checks



from __future__ import annotations

import os
from typing import TYPE_CHECKING

from checkov.terraform.module_loading.loaders.git_loader import GenericGitLoader

if TYPE_CHECKING:
    from checkov.terraform.module_loading.module_params import ModuleParams


class BitbucketAccessTokenLoader(GenericGitLoader):
    def discover(self, module_params: ModuleParams) -> None:
        self.module_source_prefix = "bitbucket.org"
        module_params.username = os.getenv('BITBUCKET_USERNAME', '')
        app_password = os.getenv('BITBUCKET_APP_PASSWORD', '')
        module_params.token = os.getenv('BITBUCKET_TOKEN', '')
        if module_params.token:
            module_params.username = "x-token-auth"
        elif module_params.username and app_password:
            module_params.token = app_password


loader = BitbucketAccessTokenLoader()



from typing import Any, Dict

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck


class KubeControllerManagerServiceAccountPrivateKeyFile(BaseK8sContainerCheck):
    def __init__(self) -> None:
        id = "CKV_K8S_110"
        name = "Ensure that the --service-account-private-key-file argument is set as appropriate"
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["command"]
        if conf.get("command"):
            if "kube-controller-manager" in conf["command"]:
                for command in conf["command"]:
                    if command.startswith("--service-account-private-key-file"):
                        file_name = command.split("=")[1]
                        extension = file_name.split(".")[1]
                        if extension == "pem":
                            return CheckResult.PASSED
                        else:
                            return CheckResult.FAILED
        return CheckResult.PASSED


check = KubeControllerManagerServiceAccountPrivateKeyFile()



from typing import Dict, List, Any

from checkov.common.util.type_forcers import force_list
from checkov.terraform.checks.data.base_check import BaseDataCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class StarActionPolicyDocument(BaseDataCheck):
    def __init__(self) -> None:
        name = 'Ensure no IAM policies documents allow "*" as a statement\'s actions'
        id = "CKV_AWS_49"
        supported_data = ("aws_iam_policy_document",)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_data=supported_data)

    def scan_data_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        """
            validates iam policy document
            https://learn.hashicorp.com/terraform/aws/iam-policy
        :param conf: aws_kms_key configuration
        :return: <CheckResult>
        """

        for statements in conf.get("statement", []):
            statements = force_list(statements)
            for statement in statements:
                if (
                    isinstance(statement, dict)
                    and statement.get("effect", ["Allow"]) in (["Allow"], [None])
                    and statement.get("actions")
                    and "*" in force_list(statement["actions"][0])
                ):
                    # effect: [None] is the default in a TF plan file
                    return CheckResult.FAILED

        return CheckResult.PASSED


check = StarActionPolicyDocument()



from checkov.common.vcs.vcs_schema import VCSSchema


class OrganizationSchema(VCSSchema):
    def __init__(self) -> None:
        schema = {
            "title": "Organization Full",
            "description": "Organization Full",
            "type": "object",
            "properties": {
                "login": {
                    "type": "string",
                    "examples": [
                        "github"
                    ]
                },
                "id": {
                    "type": "integer",
                    "examples": [
                        1
                    ]
                },
                "node_id": {
                    "type": "string",
                    "examples": [
                        "MDEyOk9yZ2FuaXphdGlvbjE\u003d"
                    ]
                },
                "url": {
                    "type": "string",
                    "format": "uri",
                    "examples": [
                        "https://api.github.com/orgs/github"
                    ]
                },
                "repos_url": {
                    "type": "string",
                    "format": "uri",
                    "examples": [
                        "https://api.github.com/orgs/github/repos"
                    ]
                },
                "events_url": {
                    "type": "string",
                    "format": "uri",
                    "examples": [
                        "https://api.github.com/orgs/github/events"
                    ]
                },
                "hooks_url": {
                    "type": "string",
                    "examples": [
                        "https://api.github.com/orgs/github/hooks"
                    ]
                },
                "issues_url": {
                    "type": "string",
                    "examples": [
                        "https://api.github.com/orgs/github/issues"
                    ]
                },
                "members_url": {
                    "type": "string",
                    "examples": [
                        "https://api.github.com/orgs/github/members{/member}"
                    ]
                },
                "public_members_url": {
                    "type": "string",
                    "examples": [
                        "https://api.github.com/orgs/github/public_members{/member}"
                    ]
                },
                "avatar_url": {
                    "type": "string",
                    "examples": [
                        "https://github.com/images/error/octocat_happy.gif"
                    ]
                },
                "description": {
                    "type": [
                        "string",
                        "null"
                    ],
                    "examples": [
                        "A great organization"
                    ]
                },
                "name": {
                    "type": "string",
                    "examples": [
                        "github"
                    ]
                },
                "company": {
                    "type": "string",
                    "examples": [
                        "GitHub"
                    ]
                },
                "blog": {
                    "type": "string",
                    "format": "uri",
                    "examples": [
                        "https://github.com/blog"
                    ]
                },
                "location": {
                    "type": "string",
                    "examples": [
                        "San Francisco"
                    ]
                },
                "email": {
                    "type": "string",
                    "format": "email",
                    "examples": [
                        "octocat@github.com"
                    ]
                },
                "twitter_username": {
                    "type": [
                        "string",
                        "null"
                    ],
                    "examples": [
                        "github"
                    ]
                },
                "is_verified": {
                    "type": "boolean",
                    "examples": [
                        True
                    ]
                },
                "has_organization_projects": {
                    "type": "boolean",
                    "examples": [
                        True
                    ]
                },
                "has_repository_projects": {
                    "type": "boolean",
                    "examples": [
                        True
                    ]
                },
                "public_repos": {
                    "type": "integer",
                    "examples": [
                        2
                    ]
                },
                "public_gists": {
                    "type": "integer",
                    "examples": [
                        1
                    ]
                },
                "followers": {
                    "type": "integer",
                    "examples": [
                        20
                    ]
                },
                "following": {
                    "type": "integer",
                    "examples": [
                        0
                    ]
                },
                "html_url": {
                    "type": "string",
                    "format": "uri",
                    "examples": [
                        "https://github.com/octocat"
                    ]
                },
                "created_at": {
                    "type": "string",
                    "format": "date-time",
                    "examples": [
                        "2008-01-14T04:33:35Z"
                    ]
                },
                "type": {
                    "type": "string",
                    "examples": [
                        "Organization"
                    ]
                },
                "total_private_repos": {
                    "type": "integer",
                    "examples": [
                        100
                    ]
                },
                "owned_private_repos": {
                    "type": "integer",
                    "examples": [
                        100
                    ]
                },
                "private_gists": {
                    "type": [
                        "integer",
                        "null"
                    ],
                    "examples": [
                        81
                    ]
                },
                "disk_usage": {
                    "type": [
                        "integer",
                        "null"
                    ],
                    "examples": [
                        10000
                    ]
                },
                "collaborators": {
                    "type": [
                        "integer",
                        "null"
                    ],
                    "examples": [
                        8
                    ]
                },
                "billing_email": {
                    "type": [
                        "string",
                        "null"
                    ],
                    "format": "email",
                    "examples": [
                        "org@example.com"
                    ]
                },
                "plan": {
                    "type": "object",
                    "properties": {
                        "name": {
                            "type": "string"
                        },
                        "space": {
                            "type": "integer"
                        },
                        "private_repos": {
                            "type": "integer"
                        },
                        "filled_seats": {
                            "type": "integer"
                        },
                        "seats": {
                            "type": "integer"
                        }
                    },
                    "required": [
                        "name",
                        "space",
                        "private_repos"
                    ]
                },
                "default_repository_permission": {
                    "type": [
                        "string",
                        "null"
                    ]
                },
                "members_can_create_repositories": {
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "examples": [
                        True
                    ]
                },
                "two_factor_requirement_enabled": {
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "examples": [
                        True
                    ]
                },
                "members_allowed_repository_creation_type": {
                    "type": "string",
                    "examples": [
                        "all"
                    ]
                },
                "members_can_create_public_repositories": {
                    "type": "boolean",
                    "examples": [
                        True
                    ]
                },
                "members_can_create_private_repositories": {
                    "type": "boolean",
                    "examples": [
                        True
                    ]
                },
                "members_can_create_internal_repositories": {
                    "type": "boolean",
                    "examples": [
                        True
                    ]
                },
                "members_can_create_pages": {
                    "type": "boolean",
                    "examples": [
                        True
                    ]
                },
                "members_can_create_public_pages": {
                    "type": "boolean",
                    "examples": [
                        True
                    ]
                },
                "members_can_create_private_pages": {
                    "type": "boolean",
                    "examples": [
                        True
                    ]
                },
                "members_can_fork_private_repositories": {
                    "type": [
                        "boolean",
                        "null"
                    ],
                    "examples": [
                        False
                    ]
                },
                "web_commit_signoff_required": {
                    "type": "boolean",
                    "examples": [
                        False
                    ]
                },
                "updated_at": {
                    "type": "string",
                    "format": "date-time"
                },
                "advanced_security_enabled_for_new_repositories": {
                    "type": "boolean",
                    "description": "Whether GitHub Advanced Security is enabled for new repositories and repositories transferred to this organization.\n\nThis field is only visible to organization owners or members of a team with the security manager role.",
                    "examples": [
                        False
                    ]
                },
                "dependabot_alerts_enabled_for_new_repositories": {
                    "type": "boolean",
                    "description": "Whether GitHub Advanced Security is automatically enabled for new repositories and repositories transferred to\nthis organization.\n\nThis field is only visible to organization owners or members of a team with the security manager role.",
                    "examples": [
                        False
                    ]
                },
                "dependabot_security_updates_enabled_for_new_repositories": {
                    "type": "boolean",
                    "description": "Whether dependabot security updates are automatically enabled for new repositories and repositories transferred\nto this organization.\n\nThis field is only visible to organization owners or members of a team with the security manager role.",
                    "examples": [
                        False
                    ]
                },
                "dependency_graph_enabled_for_new_repositories": {
                    "type": "boolean",
                    "description": "Whether dependency graph is automatically enabled for new repositories and repositories transferred to this\norganization.\n\nThis field is only visible to organization owners or members of a team with the security manager role.",
                    "examples": [
                        False
                    ]
                },
                "secret_scanning_enabled_for_new_repositories": {
                    "type": "boolean",
                    "description": "Whether secret scanning is automatically enabled for new repositories and repositories transferred to this\norganization.\n\nThis field is only visible to organization owners or members of a team with the security manager role.",
                    "examples": [
                        False
                    ]
                },
                "secret_scanning_push_protection_enabled_for_new_repositories": {
                    "type": "boolean",
                    "description": "Whether secret scanning push protection is automatically enabled for new repositories and repositories\ntransferred to this organization.\n\nThis field is only visible to organization owners or members of a team with the security manager role.",
                    "examples": [
                        False
                    ]
                },
                "secret_scanning_push_protection_custom_link_enabled": {
                    "type": "boolean",
                    "description": "Whether a custom link is shown to contributors who are blocked from pushing a secret by push protection.",
                    "examples": [
                        False
                    ]
                },
                "secret_scanning_push_protection_custom_link": {
                    "type": [
                        "string",
                        "null"
                    ],
                    "description": "An optional URL string to display to contributors who are blocked from pushing a secret.",
                    "examples": [
                        "https://github.com/test-org/test-repo/blob/main/README.md"
                    ]
                }
            },
            "required": [
                "login",
                "url",
                "id",
                "node_id",
                "repos_url",
                "events_url",
                "hooks_url",
                "issues_url",
                "members_url",
                "public_members_url",
                "avatar_url",
                "description",
                "html_url",
                "has_organization_projects",
                "has_repository_projects",
                "public_repos",
                "public_gists",
                "followers",
                "following",
                "type",
                "created_at",
                "updated_at"
            ]
        }
        super().__init__(schema=schema)


schema = OrganizationSchema()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_check import BaseResourceCheck


class FrontdoorUseWAFMode(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that Azure Front Door uses WAF in \"Detection\" or \"Prevention\" modes"
        id = "CKV_AZURE_123"
        supported_resources = ('Microsoft.Network/FrontDoorWebApplicationFirewallPolicies',)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        properties = conf.get('properties')
        if properties and isinstance(properties, dict):
            policy_settings = properties.get('policySettings')
            if policy_settings and isinstance(policy_settings, dict):
                if policy_settings.get('enabledState') == "Enabled":
                    return CheckResult.PASSED
        return CheckResult.FAILED

    def get_evaluated_keys(self) -> list[str]:
        return ["policySettings/[0]/enabledState"]


check = FrontdoorUseWAFMode()



from __future__ import annotations
from typing import Iterable
from typing_extensions import TypedDict


class EnrichedSecret(TypedDict):
    original_secret: str
    bc_check_id: str
    resource: str
    line_number: int


class SecretsCoordinator:
    __slots__ = ("_secrets", )

    def __init__(self) -> None:
        self._secrets: list[EnrichedSecret] = []

    def add_secret(self, enriched_secret: EnrichedSecret) -> None:
        # should not have duplicates? - if duplicates allowed, implementation should be changed
        # may be saved by file type first, then by key - or any other preprocessing that may help differ the secrets.
        self._secrets.append(enriched_secret)

    def get_resources(self) -> Iterable[str]:
        return [enriched_secret["resource"] for enriched_secret in self._secrets]

    def get_secrets(self) -> list[EnrichedSecret]:
        return self._secrets



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AbsNACLUnrestrictedIngress(BaseResourceCheck):
    def __init__(self, check_id, port):
        name = "Ensure no NACL allow ingress from 0.0.0.0:0 to port %d" % port
        supported_resources = ['aws_network_acl', 'aws_network_acl_rule']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=check_id, categories=categories, supported_resources=supported_resources)
        self.port = port

    def scan_resource_conf(self, conf):
        """

            Return PASS if:
            - The NACL doesn't allow unrestricted ingress to the port
            - The resource is an aws_network_acl of type 'ingress' that does not violate the check.

            Return FAIL if:
            - The NACL allows unrestricted access to the port

            Return UNKNOWN if:
            - the resource is an NACL of type 'egress', OR

        :param conf: aws_network_acl configuration
        :return: <CheckResult>
        """

        if conf.get("ingress"):
            ingress = conf.get("ingress")
            # rules are processed in numeric order
            if isinstance(ingress, list):
                entry = ingress[0]
                if isinstance(entry, list):
                    entry = ingress[0]
                    if isinstance(entry, dict) and entry.get('rule_no'):
                        ingress[0].sort(key=lambda x: x.get('rule_no'))
                elif isinstance(ingress[0], dict) and ingress[0].get('rule_no'):
                    ingress.sort(key=lambda x: x.get('rule_no'))

            for rule in ingress:
                rule_lst = rule
                if not isinstance(rule_lst, list):
                    rule_lst = [rule_lst]
                for sub_rule in rule_lst:
                    if not isinstance(sub_rule, dict):
                        return CheckResult.UNKNOWN
                    if not self.check_rule(sub_rule):
                        return CheckResult.FAILED
                    if self.check_deny_rule(sub_rule):
                        return CheckResult.PASSED
            return CheckResult.PASSED
        # maybe it's a network_acl_rule
        if conf.get("network_acl_id"):
            if not conf.get("egress") or not conf.get("egress")[0]:
                if not self.check_rule(conf):
                    return CheckResult.FAILED
            return CheckResult.PASSED

        return CheckResult.UNKNOWN

    def check_rule(self, rule):
        try:
            from_port = int(rule.get('from_port', [None])[0])
            to_port = int(rule.get('to_port', [None])[0])
        except (TypeError, ValueError):
            from_port = None
            to_port = None

        if rule.get('cidr_block'):
            if rule.get('cidr_block') == ["0.0.0.0/0"]:
                if rule.get('action') == ["allow"] or rule.get('rule_action') == ["allow"]:
                    protocol = rule.get('protocol')
                    if protocol and str(protocol[0]) == "-1":
                        return False
                    if from_port and to_port and from_port <= self.port <= to_port:
                        return False
        if rule.get('ipv6_cidr_block'):
            if rule.get('ipv6_cidr_block') == ["::/0"]:
                if rule.get('action') == ["allow"] or rule.get('rule_action') == ["allow"]:
                    protocol = rule.get('protocol')
                    if protocol and str(protocol[0]) == "-1":
                        return False
                    if from_port and to_port and from_port <= self.port <= to_port:
                        return False
        return True

    def check_deny_rule(self, rule):
        try:
            from_port = int(rule.get('from_port', [None])[0])
            to_port = int(rule.get('to_port', [None])[0])
        except (TypeError, ValueError):
            from_port = None
            to_port = None

        if rule.get('cidr_block') == ["0.0.0.0/0"]:
            if rule.get('action') == ["deny"] or rule.get('rule_action') == ["deny"]:
                protocol = rule.get('protocol')
                if protocol and str(protocol[0]) == "-1":
                    return True
                if from_port and to_port and from_port <= self.port <= to_port:
                    return True
        if rule.get('ipv6_cidr_block') == ["::/0"]:
            if rule.get('action') == ["deny"] or rule.get('rule_action') == ["deny"]:
                protocol = rule.get('protocol')
                if protocol and str(protocol[0]) == "-1":
                    return True
                if from_port and to_port and from_port <= self.port <= to_port:
                    return True
        return False



from __future__ import annotations

import io
import itertools
import logging
import os
import subprocess  # nosec
import tempfile
import threading
from typing import Any, Type, TYPE_CHECKING
import yaml

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.graph.checks_infra.registry import BaseRegistry
from checkov.common.typing import LibraryGraphConnector
from checkov.common.graph.graph_builder.consts import GraphSource
from checkov.common.images.image_referencer import fix_related_resource_ids, Image
from checkov.common.output.report import Report
from checkov.common.parallelizer.parallel_runner import parallel_runner
from checkov.common.runners.base_runner import BaseRunner, filter_ignored_paths
from checkov.helm.image_referencer.manager import HelmImageReferencerManager
from checkov.helm.registry import registry
from checkov.kubernetes.graph_builder.local_graph import KubernetesLocalGraph
from checkov.kubernetes.runner import Runner as k8_runner, handle_timeout, _KubernetesContext, _KubernetesDefinitions
from checkov.runner_filter import RunnerFilter
import signal

if TYPE_CHECKING:
    from checkov.kubernetes.graph_manager import KubernetesGraphManager
    from networkx import DiGraph


class K8sHelmRunner(k8_runner):
    def __init__(
        self,
        graph_class: Type[KubernetesLocalGraph] = KubernetesLocalGraph,
        db_connector: LibraryGraphConnector | None = None,
        source: str = GraphSource.KUBERNETES,
        graph_manager: KubernetesGraphManager | None = None,
        external_registries: list[BaseRegistry] | None = None
    ) -> None:
        self.check_type = CheckType.HELM
        super().__init__(graph_class, db_connector, source, graph_manager, external_registries)
        self.chart_dir_and_meta: list[tuple[str, dict[str, Any]]] = []
        self.pbar.turn_off_progress_bar()
        self.original_root_dir = ''
        self.tmp_root_dir = ''

    def run(
        self,
        root_folder: str | None,
        external_checks_dir: list[str] | None = None,
        files: list[str] | None = None,
        runner_filter: RunnerFilter | None = None,
        collect_skip_comments: bool = True
    ) -> Report | list[Report]:
        runner_filter = runner_filter or RunnerFilter()
        report = Report(self.check_type)

        if not self.chart_dir_and_meta:
            return report
        if external_checks_dir:
            for directory in external_checks_dir:
                registry.load_external_checks(directory)
        try:
            chart_results = super().run(root_folder, external_checks_dir=external_checks_dir, runner_filter=runner_filter)

            if isinstance(chart_results, list):
                helm_report = next(
                    chart_result for chart_result in chart_results if chart_result.check_type == self.check_type
                )
                sca_image_report = next(
                    chart_result for chart_result in chart_results if chart_result.check_type == CheckType.SCA_IMAGE
                )
            else:
                helm_report = chart_results
                sca_image_report = None

            if root_folder is not None:
                fix_report_paths(report=helm_report, tmp_dir=root_folder)
                if self.original_root_dir:
                    fix_related_resource_ids(report=sca_image_report, tmp_dir=self.original_root_dir)
                else:
                    fix_related_resource_ids(report=sca_image_report, tmp_dir=root_folder)

            return chart_results
        except Exception:
            logging.warning(f"Failed to run Kubernetes runner on charts {self.chart_dir_and_meta}", exc_info=True)
            # with tempfile.TemporaryDirectory() as save_error_dir:
            # TODO this will crash the run when target_dir gets cleaned up, since it no longer exists
            # we either need to copy or find another way to extract whatever we want to get from this (the TODO below)
            # logging.debug(
            #    f"Error running k8s scan on {chart_meta['name']}. Scan dir: {target_dir}. Saved context dir: {save_error_dir}")
            # shutil.move(target_dir, save_error_dir)

            # TODO: Export helm dependencies for the chart we've extracted in chart_dependencies
            return report

    def get_image_report(self, root_folder: str | None, runner_filter: RunnerFilter) -> Report | None:
        if not self.graph_manager:
            return None
        return self.check_container_image_references(
            graph_connector=self.graph_manager.get_reader_endpoint(),
            root_path=self.original_root_dir,
            runner_filter=runner_filter,
        )

    def extract_images(
            self,
            graph_connector: DiGraph | None = None,
            definitions: None = None,
            definitions_raw: dict[str, list[tuple[int, str]]] | None = None
    ) -> list[Image]:
        if not graph_connector:
            # should not happen
            return []

        manager = HelmImageReferencerManager(graph_connector=graph_connector, original_root_dir=self.original_root_dir,
                                             temp_root_dir=self.tmp_root_dir)
        images = manager.extract_images_from_resources()

        return images


class Runner(BaseRunner[_KubernetesDefinitions, _KubernetesContext, "KubernetesGraphManager"]):
    check_type: str = CheckType.HELM  # noqa: CCE003  # a static attribute
    helm_command = 'helm'  # noqa: CCE003  # a static attribute
    system_deps = True  # noqa: CCE003  # a static attribute

    def __init__(self) -> None:
        super().__init__()
        self.file_names = ['Chart.yaml']
        self.target_folder_path = ''
        self.root_folder = ''
        self.runner_filter: "RunnerFilter | None" = None

    def get_k8s_target_folder_path(self) -> str:
        return self.target_folder_path

    @staticmethod
    def parse_helm_chart_details(chart_path: str) -> dict[str, Any] | None:
        with open(f"{chart_path}/Chart.yaml", 'r') as chartyaml:
            try:
                chart_meta: dict[str, Any] = yaml.safe_load(chartyaml)
            except (yaml.YAMLError, UnicodeDecodeError):
                logging.info(f"Failed to load chart metadata from {chart_path}/Chart.yaml.", exc_info=True)
                return None
        return chart_meta

    def check_system_deps(self) -> str | None:
        # Ensure local system dependancies are available and of the correct version.
        # Returns framework names to skip if deps fail.
        logging.info(f"Checking necessary system dependancies for {self.check_type} checks.")
        try:
            proc = subprocess.Popen([self.helm_command, 'version'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)  # nosec
            o, e = proc.communicate()
            oString = str(o, 'utf-8')
            if "Version:" in oString:
                helmVersionOutput = oString[oString.find(':') + 2: oString.find(',') - 1]
                if "v3" in helmVersionOutput:
                    logging.info(f"Found working version of {self.check_type} dependancies: {helmVersionOutput}")
                    return None
            else:
                return self.check_type
        except Exception:
            logging.info(f"Error running necessary tools to process {self.check_type} checks.")

        return self.check_type

    @staticmethod
    def _parse_output(target_dir: str, output: bytes) -> None:
        output_str = str(output, 'utf-8')
        reader = io.StringIO(output_str)
        cur_source_file = None
        cur_writer = None
        last_line_dashes = False
        line_num = 1
        for s in reader:
            s = s.rstrip()
            if s == '---':
                last_line_dashes = True
                continue

            if last_line_dashes:
                # The next line should contain a "Source" comment saying the name of the file it came from
                # So we will close the old file, open a new file, and write the dashes from last iteration plus this line

                if not s.startswith('# Source: '):
                    raise Exception(f'Line {line_num}: Expected line to start with # Source: {s}')
                source = s[10:]
                if source != cur_source_file:
                    if cur_writer:
                        cur_writer.close()
                    file_path = os.path.join(target_dir, source)
                    parent = os.path.dirname(file_path)
                    os.makedirs(parent, exist_ok=True)
                    cur_source_file = source
                    cur_writer = open(os.path.join(target_dir, source), 'a')
                if cur_writer:
                    cur_writer.write('---' + os.linesep)
                    cur_writer.write(s + os.linesep)

                last_line_dashes = False
            else:
                if s.startswith('# Source: '):
                    raise Exception(f'Line {line_num}: Unexpected line starting with # Source: {s}')

                if not cur_writer:
                    continue
                else:
                    cur_writer.write(s + os.linesep)

            line_num += 1

        if cur_writer:
            cur_writer.close()

    @staticmethod
    def _get_target_dir(chart_item: tuple[str, dict[str, Any]], root_folder: str, target_folder_path: str) -> str | None:
        (chart_dir, chart_meta) = chart_item
        target_dir = chart_dir.replace(root_folder, f'{target_folder_path}/')
        target_dir.replace("//", "/")
        chart_name = chart_meta.get('name', chart_meta.get('Name'))
        if not chart_name:
            logging.info(
                f"Error parsing chart located {chart_dir}, chart has no name available",
                exc_info=True,
            )
            return None
        if target_dir.endswith('/'):
            target_dir = target_dir[:-1]
        if target_dir.endswith(chart_name):
            target_dir = target_dir[:-len(chart_name)]
        return target_dir

    @staticmethod
    def get_binary_output_from_directory(chart_dir: str, target_dir: str, helm_command: str,
                                         runner_filter: RunnerFilter, timeout: int = 3600) \
            -> tuple[bytes, tuple[str, dict[str, Any]]] | tuple[None, None]:
        chart_meta = Runner.parse_helm_chart_details(chart_dir)
        chart_item = (chart_dir, chart_meta or {})
        return Runner.get_binary_output(chart_item, target_dir, helm_command, runner_filter, timeout)

    @staticmethod
    def get_binary_output(
        chart_item: tuple[str, dict[str, Any]], target_dir: str, helm_command: str, runner_filter: RunnerFilter, timeout: int = 3600
    ) -> tuple[bytes, tuple[str, dict[str, Any]]] | tuple[None, None]:
        (chart_dir, chart_meta) = chart_item
        if not isinstance(chart_meta, dict):
            logging.error(f"invalid chart meta {chart_meta}")
            return None, None
        chart_name = chart_meta.get('name', chart_meta.get('Name'))
        chart_version = chart_meta.get('version', chart_meta.get('Version'))
        logging.info(
            f"Processing chart found at: {chart_dir}, name: {chart_name}, version: {chart_version}")
        # dependency list is nicer to parse than dependency update.
        try:
            helm_binary_list_chart_deps = subprocess.Popen([helm_command, 'dependency', 'list', chart_dir], stdout=subprocess.PIPE, stderr=subprocess.PIPE)  # nosec
            o, e = helm_binary_list_chart_deps.communicate()
        except Exception:
            logging.error('Error run helm command', exc_info=True)
            return None, None
        logging.debug(
            f"Ran helm command to get dependency output. Chart: {chart_name}. dir: {target_dir}. Output: {str(o, 'utf-8')}. Errors: {str(e, 'utf-8')}")
        if e:
            if "Warning: Dependencies" in str(e, 'utf-8'):
                logging.warning(
                    f"V1 API chart without Chart.yaml dependancies. Skipping chart dependancy list for {chart_name} at dir: {chart_dir}. Working dir: {target_dir}. Error details: {str(e, 'utf-8')}")
            else:
                logging.warning(
                    f"Error processing helm dependancies for {chart_name} at source dir: {chart_dir}. Working dir: {target_dir}. Error details: {str(e, 'utf-8')}")

        helm_command_args = [helm_command, 'template', '--dependency-update', chart_dir]
        if runner_filter.var_files:
            for var in runner_filter.var_files:
                helm_command_args.append("--values")
                helm_command_args.append(var)

        if threading.current_thread() is threading.main_thread():
            signal.signal(signal.SIGALRM, handle_timeout)
            signal.alarm(timeout)

        try:
            # --dependency-update needed to pull in deps before templating.
            proc = subprocess.Popen(helm_command_args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)  # nosec
            o, e = proc.communicate()
            if threading.current_thread() is threading.main_thread():
                signal.alarm(0)
            if e:
                logging.warning(
                    f"Error processing helm chart {chart_name} at dir: {chart_dir}. Working dir: {target_dir}. Error details: {str(e, 'utf-8')}")
                return None, None
            logging.debug(
                f"Ran helm command to template chart output. Chart: {chart_name}. dir: {target_dir}. Output: {str(o, 'utf-8')}. Errors: {str(e, 'utf-8')}")
            logging.info(f'Done helm run for: {chart_dir}')
            return o, chart_item

        except Exception as e:
            if threading.current_thread() is threading.main_thread():
                signal.alarm(0)
            if isinstance(e, TimeoutError):
                logging.info(
                    f"Error processing helm chart {chart_name} at dir: {chart_dir}. Working dir: {target_dir}. got timeout"
                )
            else:
                logging.info(
                    f"Error processing helm chart {chart_name} at dir: {chart_dir}. Working dir: {target_dir}.",
                    exc_info=True,
                )
            return None, None

    @staticmethod
    def _convert_chart_to_k8s(
        chart_item: tuple[str, dict[str, Any]],
        root_folder: str,
        target_folder_path: str,
        helm_command: str,
        runner_filter: RunnerFilter,
    ) -> None:
        target_dir = Runner._get_target_dir(chart_item, root_folder, target_folder_path)
        if not target_dir:
            return

        o, _ = Runner.get_binary_output(chart_item, target_folder_path, helm_command, runner_filter)
        if o is None:
            return

        try:
            Runner._parse_output(target_dir, o)
        except Exception:
            (chart_dir, chart_meta) = chart_item
            chart_name = chart_meta.get('name', chart_meta.get('Name'))
            logging.info(
                f"Error parsing output {chart_name} at dir: {chart_dir}. Working dir: {target_dir}.",
                exc_info=True,
            )

    @staticmethod
    def _get_chart_dir_and_meta(
        root_folder: str | None, files: list[str] | None, runner_filter: RunnerFilter
    ) -> list[tuple[str, dict[str, Any]]]:
        chart_directories = find_chart_directories(root_folder, files, runner_filter.excluded_paths)
        chart_dir_and_meta = list(parallel_runner.run_function(
            lambda cd: (cd, Runner.parse_helm_chart_details(cd)), chart_directories))
        # remove parsing failures
        cleaned_chart_dir_and_meta = [(chart_dir, meta) for chart_dir, meta in chart_dir_and_meta if meta]
        return cleaned_chart_dir_and_meta

    @staticmethod
    def _get_processed_chart_dir_and_meta(
        chart_dir_and_meta: list[tuple[str, dict[str, Any]]], root_folder: str
    ) -> list[tuple[str, dict[str, Any]]]:
        processed_chart_dir_and_meta = []
        for chart_dir, chart_meta in chart_dir_and_meta:
            processed_chart_dir_and_meta.append((chart_dir.replace(root_folder, ""), chart_meta))
        return processed_chart_dir_and_meta

    def convert_helm_to_k8s(
        self, root_folder: str | None, files: list[str] | None, runner_filter: RunnerFilter
    ) -> list[tuple[Any, dict[str, Any]]]:
        self.root_folder = root_folder or ""
        self.runner_filter = runner_filter
        self.target_folder_path = tempfile.mkdtemp()
        chart_dir_and_meta = Runner._get_chart_dir_and_meta(self.root_folder, files, runner_filter)

        list(
            parallel_runner.run_function(
                lambda cd: Runner._convert_chart_to_k8s(
                    chart_item=cd,
                    root_folder=self.root_folder,
                    target_folder_path=self.target_folder_path,
                    helm_command=self.helm_command,
                    runner_filter=runner_filter,
                ),
                chart_dir_and_meta,
            )
        )
        return Runner._get_processed_chart_dir_and_meta(chart_dir_and_meta, self.root_folder)

    def run(
        self,
        root_folder: str | None,
        external_checks_dir: list[str] | None = None,
        files: list[str] | None = None,
        runner_filter: RunnerFilter | None = None,
        collect_skip_comments: bool = True
    ) -> Report | list[Report]:
        runner_filter = runner_filter or RunnerFilter()
        if not runner_filter.show_progress_bar:
            self.pbar.turn_off_progress_bar()

        k8s_runner = K8sHelmRunner()
        k8s_runner.chart_dir_and_meta = self.convert_helm_to_k8s(root_folder, files, runner_filter)
        k8s_runner.original_root_dir = str(root_folder)
        k8s_runner.tmp_root_dir = self.get_k8s_target_folder_path()
        report = k8s_runner.run(self.get_k8s_target_folder_path(), external_checks_dir=external_checks_dir, runner_filter=runner_filter)
        self.graph_manager = k8s_runner.graph_manager
        return report


def fix_report_paths(report: Report, tmp_dir: str) -> None:
    for check in itertools.chain(report.failed_checks, report.passed_checks):
        check.repo_file_path = check.repo_file_path.replace(tmp_dir, '', 1)
    report.resources = {r.replace(tmp_dir, '', 1) for r in report.resources}


def get_skipped_checks(entity_conf: dict[str, Any]) -> list[dict[str, str]]:
    skipped = []
    metadata = {}
    if not isinstance(entity_conf, dict):
        return skipped
    if entity_conf["kind"] == "containers" or entity_conf["kind"] == "initContainers":
        metadata = entity_conf["parent_metadata"]
    else:
        if "metadata" in entity_conf.keys():
            metadata = entity_conf["metadata"]
    if "annotations" in metadata.keys() and metadata["annotations"] is not None:
        for key in metadata["annotations"].keys():
            skipped_item = {}
            if "checkov.io/skip" in key or "bridgecrew.io/skip" in key:
                if "CKV_K8S" in metadata["annotations"][key]:
                    if "=" in metadata["annotations"][key]:
                        (skipped_item["id"], skipped_item["suppress_comment"]) = metadata["annotations"][key].split("=")
                    else:
                        skipped_item["id"] = metadata["annotations"][key]
                        skipped_item["suppress_comment"] = "No comment provided"
                    skipped.append(skipped_item)
                else:
                    logging.info(f"Parse of Annotation Failed for {metadata['annotations'][key]}: {entity_conf}")
                    continue
    return skipped


def find_chart_directories(root_folder: str | None, files: list[str] | None, excluded_paths: list[str]) -> list[str]:
    chart_directories = []
    if not excluded_paths:
        excluded_paths = []
    if files:
        logging.info('Running with --file argument; checking for Helm Chart.yaml files')
        for file in files:
            if os.path.basename(file) == 'Chart.yaml':
                chart_directories.append(os.path.dirname(file))

    if root_folder:
        for root, d_names, f_names in os.walk(root_folder):
            filter_ignored_paths(root, d_names, excluded_paths)
            filter_ignored_paths(root, f_names, excluded_paths)
            if 'Chart.yaml' in f_names:
                chart_directories.append(root)

    return chart_directories



from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class RedshiftClusterAutoSnap(BaseResourceNegativeValueCheck):
    def __init__(self):
        """
        NIST.800-53.r5 CP-10, NIST.800-53.r5 CP-6, NIST.800-53.r5 CP-6(1), NIST.800-53.r5 CP-6(2), NIST.800-53.r5 CP-9,
        NIST.800-53.r5 SC-5(2), NIST.800-53.r5 SC-7(10), NIST.800-53.r5 SI-13(5)
        Amazon Redshift clusters should have automatic snapshots enabled
        """
        name = "Ensure Amazon Redshift clusters should have automatic snapshots enabled"
        id = "CKV_AWS_343"
        supported_resources = ['aws_redshift_cluster']
        categories = [CheckCategories.BACKUP_AND_RECOVERY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_attribute_result=CheckResult.PASSED)

    def get_inspected_key(self):
        return "automated_snapshot_retention_period"

    def get_forbidden_values(self):
        return [0]


check = RedshiftClusterAutoSnap()



from __future__ import annotations

from checkov.common.graph.graph_builder import EncryptionTypes
from checkov.common.graph.graph_builder.graph_components.generic_resource_encryption_base import GenericResourceEncryptionBase
from checkov.common.util.data_structures_utils import get_empty_list_str


class GenericResourceEncryption(GenericResourceEncryptionBase):
    def __init__(
        self,
        resource_type: str,
        attribute_values_map: dict[str, list[bool] | list[str]],
        enabled_by_default: bool = False,
    ) -> None:
        super().__init__(resource_type,
                         attribute_values_map,
                         enabled_by_default,
                         node_to_node_encryption="NodeToNodeEncryptionOptions")
        if self.resource_type.startswith("AWS::"):
            self.default_description = EncryptionTypes.DEFAULT_KMS.value


# This map allows dynamically creating the check for each resource type based on GenericResourceEncryption.
# Please check out the constructor to understand all the edge cases.
ENCRYPTION_BY_RESOURCE_TYPE: dict[str, GenericResourceEncryption] = {
    "AWS::ECR::Repository": GenericResourceEncryption(
        "AWS::ECR::Repository",
        {
            "EncryptionConfiguration.EncryptionType": [EncryptionTypes.AES256.value, EncryptionTypes.KMS_VALUE.value],
            "EncryptionConfiguration.KmsKey": get_empty_list_str(),
        },
    ),
    "AWS::Neptune::DBCluster": GenericResourceEncryption(
        "AWS::Neptune::DBCluster", {"StorageEncrypted": [True], "KmsKeyId": get_empty_list_str()}
    ),
    "AWS::EFS::FileSystem": GenericResourceEncryption("AWS::EFS::FileSystem", {"Encrypted": [True], "KmsKeyId": get_empty_list_str()}),
    "AWS::EC2::Volume": GenericResourceEncryption("AWS::EC2::Volume", {"Encrypted": [True], "KmsKeyId": get_empty_list_str()}),
    "AWS::ElastiCache::ReplicationGroup": GenericResourceEncryption(
        "AWS::ElastiCache::ReplicationGroup", {"AtRestEncryptionEnabled": [True], "KmsKeyId": ["arn"]}
    ),
    "AWS::Elasticsearch::Domain": GenericResourceEncryption(
        "AWS::Elasticsearch::Domain",
        {"EncryptionAtRestOptions.Enabled": [True], "EncryptionAtRestOptions.KmsKeyId": get_empty_list_str(), "NodeToNodeEncryptionOptions.Enabled": [True]},
    ),
    "AWS::MSK::Cluster": GenericResourceEncryption(
        "AWS::MSK::Cluster", {"EncryptionInfo.EncryptionAtRest.DataVolumeKMSKeyId": get_empty_list_str()}
    ),
    "AWS::DocDB::DBCluster": GenericResourceEncryption(
        "AWS::DocDB::DBCluster", {"StorageEncrypted": [True], "KmsKeyId": get_empty_list_str()}
    ),
    "AWS::CodeBuild::Project": GenericResourceEncryption("AWS::CodeBuild::Project", {"EncryptionKey": get_empty_list_str()}),
    "AWS::CodeBuild::ReportGroup": GenericResourceEncryption(
        "AWS::CodeBuild::ReportGroup",
        {
            "ExportConfig.S3Destination.EncryptionDisabled": [False],
            "ExportConfig.S3Destination.EncryptionKey": get_empty_list_str(),
        },
    ),
    "AWS::Athena::WorkGroup": GenericResourceEncryption(
        "AWS::Athena::WorkGroup",
        {
            "WorkGroupConfiguration.ResultConfiguration.EncryptionConfiguration.EncryptionOption": [
                "SSE_S3",
                "SSE_KMS",
                "CSE_KMS",
            ],
            "WorkGroupConfiguration.ResultConfiguration.EncryptionConfiguration.KmsKey": get_empty_list_str(),
        },
    ),
    "AWS::Kinesis::Stream": GenericResourceEncryption(
        "AWS::Kinesis::Stream", {"StreamEncryption.EncryptionType": [EncryptionTypes.KMS_VALUE.value], "StreamEncryption.KeyId": get_empty_list_str()}
    ),
    "AWS::EKS::Cluster": GenericResourceEncryption("AWS::EKS::Cluster", {"EncryptionConfig.Provider.KeyArn": get_empty_list_str()}),
    "AWS::DynamoDB::Table": GenericResourceEncryption(
        "AWS::DynamoDB::Table",
        {"SSESpecification.SSEEnabled": [True], "SSESpecification.KMSMasterKeyId": get_empty_list_str(), "SSESpecification.SSEType": get_empty_list_str()},
        enabled_by_default=True,
    ),
    "AWS::RDS::DBCluster": GenericResourceEncryption("AWS::RDS::DBCluster", {"StorageEncrypted": [True], "KmsKeyId": get_empty_list_str()}),
    "AWS::RDS::GlobalCluster": GenericResourceEncryption("AWS::RDS::GlobalCluster", {"StorageEncrypted": [True]}),
    "AWS::S3::Bucket": GenericResourceEncryption(
        "AWS::S3::Bucket",
        {
            "BucketEncryption.ServerSideEncryptionConfiguration.ServerSideEncryptionByDefault.SSEAlgorithm": [
                EncryptionTypes.AWS_KMS_VALUE.value,
                EncryptionTypes.AES256.value,
            ],
            "server_side_encryption_configuration.rule.apply_server_side_encryption_by_default.KMSMasterKeyID": get_empty_list_str(),
        },
    ),
    "AWS::Logs::LogGroup": GenericResourceEncryption(
        "AWS::Logs::LogGroup", {"KmsKeyId": get_empty_list_str()}, enabled_by_default=True
    ),
    "AWS::CloudTrail::Trail": GenericResourceEncryption("AWS::CloudTrail::Trail", {"KMSKeyId": get_empty_list_str()}),
    "AWS::DAX::Cluster": GenericResourceEncryption("AWS::DAX::Cluster", {"SSESpecification.SSEEnabled": [True]}),
    "AWS::Redshift::Cluster": GenericResourceEncryption("AWS::Redshift::Cluster", {"Encrypted": [True], "KmsKeyId": get_empty_list_str()}),
    "AWS::SNS::Topic": GenericResourceEncryption("AWS::SNS::Topic", {"KmsMasterKeyId": get_empty_list_str()}),
    "AWS::SQS::Queue": GenericResourceEncryption("AWS::SQS::Queue", {"KmsMasterKeyId": get_empty_list_str()}),
    "AWS::RDS::DBInstance": GenericResourceEncryption("AWS::RDS::DBInstance", {"StorageEncrypted": [True]})
}
